//CHAPTER 5, SOLUTION 1

import TryGeometry.Point; // Import the Point class defined in the chapter.
// This is a slightly refined version of the Geometry.Point class used 
// in the chapter. It adds the getX() and getY() methods, where we need them.

public class Rectangle
{
  private Point topLeft;
  private Point bottomRight;

  // Constructors
  public Rectangle(double x1, double y1, double x2, double y2)
  {
    // Ensure topleft uses the smaller of x1,x2 and y1,y2
    // and topright uses the larger of x1,x2 and y1,y2
    topLeft = new Point(Math.min(x1,x2),Math.min(y1,y2));
    bottomRight = new Point(Math.max(x1,x2),Math.max(y1,y2));
  }

  public Rectangle(Point tl, Point br)
  {
    this(tl.getX(), tl.getY(), br.getX(), br.getY());
  }

  public Rectangle(Rectangle rect)
  {
    topLeft = new Point(rect.topLeft);
    bottomRight = new Point(rect.bottomRight);
  }

  // Methods:
  public Rectangle encloses(Rectangle rect)
  {
    // Return the rectangle defined by the minimum and maximum x,y
    return new Rectangle(Math.min(topLeft.getX(), rect.topLeft.getX()),
                         Math.min(topLeft.getY(), rect.topLeft.getY()),
                         Math.max(topLeft.getX(), rect.topLeft.getX()),
                         Math.max(topLeft.getY(), rect.topLeft.getY()));
    
  }

  public String toString()
  {
    return "Rectangle: " + topLeft + " , "+ bottomRight;  
  }
  // A method to output the positions of the points defining a Rectangle:
  public void show()
  {
    System.out.println(toString());
  }

  public static void main(String args[])
  {
    Rectangle[] rects = new Rectangle[4];
    Rectangle enclosing;
    
    // Initialize the rectangles as arbitrary sizes and at arbitrary positions:
    for(int i=0 ; i<rects.length ; i++)
    {
      rects[i] = new Rectangle(Math.random()*100, Math.random()*100,
					                     Math.random()*100, Math.random()*100);
      System.out.print("Coords of rectangle " + i + " are: ");
      rects[i].show();
    }

    // Initialize the enclosing rectangle:
    enclosing = rects[0];
    
    // Compare it with all the other rectangles and for each comparison
    // increase its size so as to enclose all the rectangles:
    for(int i=1 ; i<rects.length ; i++)
    {
      enclosing = enclosing.encloses(rects[i]);
    }
    System.out.println("\nCoords of Enclosing rectangle are ");
    enclosing.show();
  }
}


