//CHAPTER 8, SOLUTION 1

import java.io.*;
import java.util.Random;

public class TwinDates
{
	public static void main(String[]args)
	{
    final int dateCount = 10;          // Number of dates to be output
		final int[]daysInMonth = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
	  Random random = new Random();      // Random number generator

    // Arrays holding corresponding day-month-year for a date
  	int[]day = new int[dateCount];     // Each element is day value in month from 1 upwards
    int[]month = new int[dateCount];   // Each element is month value, 1 to 12
  	int[]year = new int[dateCount];    // Years are from 1900 to 1999						 

  	// Generate a set of arbitrary dates:
  	for(int i = 0 ; i<dateCount ; i++)
  	{
	  	month[i] = random.nextInt(12) + 1 ;     // Value 1 to 12
  		year[i] = random.nextInt(200) + 1900;	  // 20th & 21st century years.

	  	if(isLeapYear(year[i]) && (month[i] == 2))	    // If it's February in a leap year:
	  		day[i] = random.nextInt(29)+1;        // Day value 1 to 29
	  	else					   // Otherwise we can use the daysInMonth array unaltered:
	  		day[i] = random.nextInt(daysInMonth[month[i]-1])+1;

  		// Display the date as an array of integers when running Dates.java: 
  		System.out.println(day[i] + "," + month[i] + "," + year[i]);
  	}



    // Create the files for the dates
		try
		{
			String textFileName = "Dates.txt";				// Name the of the text output file
      String binFileName = "Dates.bin";         // Name of the binary output file

      // Constants for generating date strings
			String[]monthSet = {"January", "February", "March", "April", "May", 
			"June", "July", "August", "September","October", "November", "December"};
			String ending = "th";


      // Create output streams to the files
			BufferedWriter out1=new BufferedWriter(new FileWriter(createFile(textFileName)));
      DataOutputStream out2 = new DataOutputStream(
                               new BufferedOutputStream(
                                new FileOutputStream(createFile(binFileName))));			
      // Create the date strings and write them to the file
			for(int i = 0 ; i<dateCount ; i++)
			{
				// Adjust day ending for 1st, 2nd, 3rd, 4th etc. of the month:
				if((day[i] > 10) && (day[i] < 14))
					ending = "th";
				else
					switch(day[i]%10)
					{
						case 1:
							ending = "st";
							break;
						case 2:
							ending = "nd";
							break;
						case 3:
							ending = "rd";
							break;
						default:
							ending = "th";
							break;
					}
				// Output the date as a string literal in the Dates.txt file:
				out1.write(day[i] + ending + " " + monthSet[month[i]-1] + " " + year[i] + "\n" );

        // Output binary values to the Dates.bin file
				out2.writeInt(month[i]);
				out2.writeInt(day[i]);
				out2.writeInt(year[i]);
			}
			out1.close();
			out2.close();
		}

		catch(IOException e)
		{
			System.out.println("Error writing the file " + e);
		}
	}

  // Returns true if argument represents a leap year
	private static boolean isLeapYear(int year)
	{ return ((year % 4 == 0) && (year % 100 != 0))||((year % 4 == 0) && (year % 100 == 0) && ((year/100) %4 ==0)); }

  // Create a file for a given file name
  private static File createFile(String fileName) throws IOException
  {
			File output = new File(fileName);
			output.createNewFile();						  // Create new file if it does not yet exists:

			if(!output.isFile())						    // Verify the file exists:
			{
				System.out.println("Creating "+output.getPath()+" failed.");
        System.exit(1);
			}
			return output;
  }
}
