import java.io.*;
// The file names can appear as command line arguments - the from file first.
public class CopyFile
{
  // We can share a keyboard stream between methods:
  static BufferedReader keyboard = new BufferedReader(new InputStreamReader(System.in)); 

  public static void main(String args[])
  {
    String fileInName = null;                             // Stores name of file to be copied.
    String fileOutName = null;                            // Store name of file copy.

    // Get the names of files to copy from/to command line:
    try
    {
      switch(args.length)
      {
	case 2:			                          // If both files are specified on command line,
	  fileInName = args[0];	                          // store the references to them.
	  fileOutName = args[1];
	  break;

	case 1:                                           // If only one file is specified,
	  fileInName = args[0];                           // assume it's to file to copy from.
	  System.out.println("Name of copied file is:");
	  fileOutName = new String(keyboard.readLine());  // Get the other file name.
	  break;

	case 0:                                           // If no filenames are specified
	  System.out.println("File to copy is:");         // get both from the keyboard.
	  fileInName = new String(keyboard.readLine());
	  System.out.println("Name of copied file is:");
	  fileOutName = new String(keyboard.readLine());
	  break;
      }
    }

    // If we didn't get the filenames, exit:
    catch(IOException e)
    {
      System.err.println("Error reading file names");
      System.err.println(e);
      System.exit(-1);
    }

    // Display the filenames we have:
    System.out.println("Input is from "+fileInName);
    System.out.println("This will be copied to "+fileOutName);

    // Copy the file:
    try
    {
      copy(fileInName,fileOutName);
    }

    // If an error occurred, display error information:
    catch(IOException e)
    {
      System.err.println("IO error on occurred in copy operation.");
      System.err.println(e);
      System.exit(1);                                  // End the program.
    }
  }

  // Method to copy one file to another.
  // First argument is the input file name, second is the output file name:
  public static void copy(String fileInName, String fileOutName) throws IOException
  {
    File fileIn = new File(fileInName);                 // File object for input file.
    File fileOut = new File(fileOutName);               // File object for output file.
    FileInputStream inStream = null;                    // Input file stream.
    FileOutputStream outStream = null;           	// Output file stream.

    // If the input file does not exist, say so and exit:
    if(!fileIn.exists())
    {
      System.err.println("File "+fileIn+" does not exist");
      return;
    }

    // If the output file does exist, ask whether it should be overwritten:
    if(fileOut.exists())
    {
      System.out.println("Do you wish to overwrite "+ fileOut +"(y/n)?");
      switch(keyboard.read())                          	// Read a character:
      {
	case 'y':
	case 'Y':
	  break;
	default:
	  System.err.println(fileOut + " not overwritten - copy aborted.");
	  return;
      }
    }

    // Copy the file...
    try
    {
      inStream = new FileInputStream(fileIn);           // Create the input stream.
      outStream = new FileOutputStream(fileOut);        // Create the output stream.
      int bytesRead = 0;                                // Number of bytes read.
      byte[] buffer = new byte[1024];                   // 1k byte input buffer.


      while((bytesRead = inStream.read(buffer)) != -1)	// Read until EOF.
      {
        System.out.println("Read "+bytesRead+" bytes.");
	outStream.write(buffer,0,bytesRead);            // Write the bytes to the output.
        System.out.println("Copied "+bytesRead+" bytes.");
      }
    }

    // Catch any exceptions - report and rethrow for caller:
    catch(IOException e)
    {
      System.err.println("Error occured during copy");
      System.err.println(e);
      throw e;                                          // Rethrow the exception.
    }

    // Ensure the input and output streams are closed if they were opened:
    finally
    {
      if(inStream != null)                              // If there is an input stream,
      {
	try
	{
	  inStream.close();                             // close it.
	}
	catch(IOException e)
	{
	  System.err.println(e);
	}
      }

      if(outStream != null)                             // If there is an output stream,
      {
	try
	{
	  outStream.close();                            // close that too.
	}
	catch(IOException e)
	{
	  System.err.println(e);
	}
      }
    }
  }
}
