import java.io.*;

public class PrimesFile
{
  static RandomAccessFile myFile = null;	// File stream.
  static boolean file = false;          	// True if file contains primes.
  static long[] primes = new long[10];  	// Array to store primes.
  static int current = 0;               	// Free element in primes array.

  // The main computation:
  public static void main(String args[])
  {
    try
    {
      // Define the file to store primes:
      File myPrimes = new File("Primes.bin");
      myFile = new RandomAccessFile(myPrimes, "rw");

      // Read the number of primes required:
      FormattedInput in = new FormattedInput(); // Keyboard input stream.
      System.out.print("Enter the number of primes required: ");
      int numPrimes = in.intRead();  		// Number of primes required.
      long count = myFile.length()/8L;  	// Number of primes in the file.
      long number = 0L;                 	// Next number to be tested.

      // Check for file contents:
      if(count == 0L)				// Nothing in the file.
      {     
        file = false;
        primes[0] = 2;             		// Seed the first prime...
        primes[1] = 3;             		// ...and the second.
        current = 2;               		// Index of next element.
        count = 2L;                		// Count of primes found - up to now.
        number = 5L;              		// Next integer to be tested.
      }

      else			// Get the next number to test - the last prime + 2:
      {   
        file = true;
        myFile.seek(myFile.length()-8);
        number = myFile.readLong()+2;
      }

      // Find additional primes required for the total of primes requested:
      for( ; count<numPrimes ; number += 2L)
      {
        if(primeTest(number))                 	// Test for a prime:
        {
          primes[current++] = number;         	// We got one!
          ++count;                            	// Increment prime count.
          if(current == primes.length)        	// Check for array full.
          {                                   	// If so write them away.
            myFile.seek(myFile.length());     	// Go to end of file.
            for(int i=0 ; i<primes.length ; i++)
              myFile.writeLong(primes[i]);    	// Write the primes.
            current = 0;                      	// Set free array element index.
            file = true;                      	// Indicate file has primes.
          }
        }
      }

      // Check if there are still primes in the array:
      if(current>0)
      { // There are - so write them to the file
        myFile.seek(myFile.length());         	// Go to the end of the file.
        for(int i = 0; i < primes.length; i++)
          myFile.writeLong(primes[i]);        	// Write the primes.
        current = 0;                          	// Set free array element index.
        file = true;                          	// Indicate file has primes.
      }
      displayNth();                           	// Optionally display a prime.
      outputPrimes(numPrimes);                	// Output all the primes.
    }

    catch(IOException e)
    {
      System.out.println(e);
    }
  }

  // Test whether a number is prime:
  static boolean primeTest(long number)
  {
    // The maximum divisor we need to try is the square root of number:
    long limit = (long)Math.ceil(Math.sqrt((double)number));
    try
    {
      if(file)             			// Check we have primes on file...
      {                    			// ... we do.
        long prime = 0L;                        // Stores prime from file.
        myFile.seek(0L);                        // Go to file start.
        long primeCount = myFile.length()/8;    // Primes on file.

        // Check using primes from file:
        for(int i=0 ; i<primeCount ; i++)
        {
          prime = myFile.readLong(); 		// Read a prime.
          if(prime>limit)
            return true;             		// No exact division - prime found.
          if(number%prime == 0)
            return false;            		// Exact division - not a prime.
        }
      }
    }

    catch(IOException e)        		// Handle read error.
    { 
      System.out.println("Exception primeTest():\n"+e);   	// Output message
      System.exit(1);                           // End the program.
    }

    // Check using primes in memory:
    for(int i=0 ; i<current ; i++)
    {
      if(primes[i]>limit)
      return true;               		// No exact division - prime found.
      if(number%primes[i] == 0)
      return false;              		// Exact division - not a prime.
    }
    return true;
  }

  // Method to output primes:
  static void outputPrimes(int numPrimes) throws IOException
  {
    // Create a buffered formatted output stream:
    FormatWriter out = new FormatWriter(
                         new BufferedWriter(
                         new FileWriter(FileDescriptor.out)), true, 12);
    myFile.seek(0);                      	// Go to file start.
    for(int i=0 ; i<numPrimes ; i++)     	// Output the primes:
    {
      long prime = myFile.readLong();
      if(i%5 == 0)
        out.print("\n");     			// After every 5th, a new line.
      out.print(prime);      			// Output the prime.
    }
    out.flush();
  }

  // Method to display any prime from the file:
  static void displayNth() throws IOException
  {
    FormattedInput keyboard = new FormattedInput();

    int primeCount = (int)(myFile.length()/8L);
    System.out.println("\nWe have the first "+ primeCount + " primes on file.");

    int nth = 0;                    		// Sequence number of prime.
    String ending = null;           		// Ending for output.

    for(;;)
    {
      System.out.println("Which of the computed primes do you wish to display?" +
                         "\n Enter 0 to end: ");

      if((nth = keyboard.intRead())==0)         // If zero was entered we are done.
        return;                                 

      nth--;                                    // We index primes from 0:

      if((nth>=primeCount) || (nth < 0))	// Must be in file.
      {
	System.out.println("The prime chosen must be between 1 and "+
							primeCount+".");
      }

      else
      {
	myFile.seek(nth*8L);			// Go to nth prime.
	long prime = myFile.readLong();
	nth++;					// Convert nth back to start from 1.

	// Format the output nicely:
	if((nth > 10) && (nth < 14))
	  ending = "th";
	else
	{
	  switch((int)nth%10)
	  {
	    case 1:
	      ending = "st";
	      break;
	    case 2:
	      ending = "nd";
	      break;
	    case 3:
	      ending = "rd";
	      break;
	    default:
	      ending = "th";
	      break;
	  }
	}
	System.out.println("The "+nth+ending+" prime is "+prime+".");
      }
    }
  }
}

