import java.io.*;
public class FormatWriter extends PrintWriter
{
  private int width = 10;      // Field width required for output

  // Basic onstructor for a default field width
  public FormatWriter(Writer out)
  {
    super(out);                // Call PrintStream constructor
  }

  // Constructor with a specified field width
  public FormatWriter(Writer out, int width)
  {
    super(out);                // Call PrintStream constructor
    this.width = width;        // Store the field width
  }

  // Constructor with autoflush option
  public FormatWriter(Writer out, boolean autoflush)
  {
    super(out, autoflush);     // Call PrintStream constructor
  }

  // Constructor with a specified field width and autoflush option 
  public FormatWriter(Writer out, boolean autoflush, int width)
  {
    super(out, autoflush);     // Call PrintStream constructor
    this.width = width;        // Store the field width
  }

  // Helper method for output
  private void output(String str)
  {
    int blanks = width - str.length();    // Number of blanks needed

    // If the length is less than the width, add blanks to the start
    if(blanks > 0)
      for (int i = 0; i < blanks; i++)
        super.print(' ');                 // Output a space
    super.print(str);                     // Use base method for output
  }

  // Output type long formatted in a given width
  public void print(long value)
  {
    output(Long.toString(value));         // Pad to width and output
  }

  // Output type double formatted in a given width
  public void print(double value)
  {
    output(Double.toString(value));       // Pad to width and output
  }

  // Output type long formatted in a given width plus a newline
  public void println(long value)
  {
    this.print(value);                     // Call current method
    super.println();                       // Call base method
  }

  // Output type double formatted in a given width plus a newline
  public void println(double value)
  {
    this.print(value);                    // Call current method
    super.println();                      // Call base method
  }

  public void setWidth(int width)
  {
    this.width = width;
  }
}

