import java.io.*;

// Program to write names to a file and sort them in ascending sequence:
public class StoreNameObjects
{

  static public void main(String[] args)
  {
    String name;                      				// Stores a name input string.
    String firstname;                 				// Stores the first name.
    String surname;                   				// Stores the surname.
    Name aName = null;                				// References a Name object.
    String namesFile = "Names.txt";   				// File name to store names.
    BufferedReader keyboard = new BufferedReader(new InputStreamReader(System.in));
    ObjectOutputStream out = null;    				// File output stream.
    ObjectInputStream in = null;      				// File input stream.

    try
    {
      out = new ObjectOutputStream(                             // Object output stream
             new BufferedOutputStream(       
                           new FileOutputStream(namesFile)));   // for names file.

      System.out.println("Enter surname followed by first name separated by a space - press Enter to end:");

      // Read names from the keyboard and write them to the file:
      // We could do more here - eliminate multiple spaces for example:

      int space;                              			// Index of space between surname and first.                                
      while((name=keyboard.readLine()).length()>0)
      {
        if((space = name.indexOf(' ')) <0)               	// Check that there is a space.
        {
          System.out.println("Enter a surname and first name separated by a space");
          continue;
        }
        surname = name.substring(0, space);              	// Extract the surname.
        firstname = name.substring(space+1);             	// Extract the first name.
        out.writeObject(new Name(firstname,surname));    	// Write the name to the file as Name object.
      }
    }

    catch(IOException e)
    {
      System.err.println("Error Writing names file\n"+e);
      System.exit(1);                                    	// End the program.
    }
 
    // Now sort the names - we will use a bubble sort as it is simple - but slow.
    // The process is to read the objects from the file and write them to another file,
    // interchanging any successive object that are not in sequence. This process
    // repeats until no interchanges are necessary - indicating the objects are in order:
    
    String fileIn = namesFile;                    		// Stores a reference to the input file.
    String fileOut = "temp.txt";                  		// Stores a reference to the output file.
    String temp;                                  		// Used for interchanging fileIn and fileOut.
    Name nextName;                                		// Stores an object read from the file.
    boolean sorted = true;                        		// Indicates file is in order.
    
    // Code added to sort names starts here:      ***
    // Loop continues until names are in order:
    do
    {
      try
      {
        out.close();                                      	// Close the names file.
        in = new ObjectInputStream(                       	// Object input stream
                 new BufferedInputStream(       
                     new FileInputStream(fileIn)));       	// for names file.
        out = new ObjectOutputStream(                     	// Object output stream.
                 new BufferedOutputStream(       
                     new FileOutputStream(fileOut)));     	// for names file.
      }

      catch(IOException e)
      {
        System.err.println("Error creating file " +e);
        System.exit(1);                                   	// End the program.
      }
      sorted = true;                                      	// Assume file is in order.

      try
      {
          aName = (Name)in.readObject();
          for(;;)		                             	// Read until EOF.
          {
            nextName = (Name)in.readObject();                   // Read the next name.

            if(aName.compareTo(nextName)<=0)                    // Check for ascending order.
	    {
	      out.writeObject(aName);                           // Write first name.
	      aName = nextName;                                 // Second name is next first.
	    }
	    else
	    {
	      out.writeObject(nextName);                        // Write second name.
	      sorted = false;                                   // Not in order.
	    }
          }
      }

      catch(EOFException e)
      { // Since this is normal you could do nothing here
	System.out.println("End of File reached\n");          	// Report end of file.
      }

      catch(IOException e)
      {
        System.err.println("Error sorting file " +e);
        System.exit(1);                                     	// End the program.
      }

      // This can be thrown by the readObject() method:
      // It usually means you are reading the wrong file, or the file is corrupted:
      catch(ClassNotFoundException e)
      {
	System.err.println(e);
        System.exit(1);
      }

      try
      {
        out.writeObject(aName);                           	// Write the last name.  
        in.close();                                             // Close the input
        out.close();                                            // and the output.

        // Switch files for next pass:
        temp = fileIn;                                         	// Save input.
        fileIn = fileOut;                                      	// Set input as last output.
        fileOut = temp;                                        	// Set output as last input.
      }

      catch(EOFException e)
      { // Since this is normal you could do nothing here
	System.out.println("End of File reached");              // Report end of file.
      }

      catch(IOException e)
      {
        System.err.println("Error sorting file " +e);
        System.exit(1);                                         // End the program.
      }		// Since we switch the files, we must ensure fileIn contains the sorted file.
    } 		// If it doesn't, the loop condition will execute an extra pass to copy it. 
    while((!sorted) || fileIn!=namesFile);

    // We have ensured that we end up with the sorted file in fileIn,
    // and that this is the names file. Therefore we can delete fileOut:
    new File(fileOut).delete();                                	// Delete the temp file.
    // Code added to sort names ends here.      ***
    
    // Close the file, display the names saved to the file by reading them back in:
    System.out.println("A list of all names in the file:");
    try
    {
      out.close();			                       	// Close the output stream.
      in = new ObjectInputStream(                              	// Object input stream
             new BufferedInputStream(       
                           new FileInputStream(namesFile)));   	// for names file.
      try
      {
        while((aName = (Name)in.readObject())!=null)		// Read until EOF.
	  System.out.println("Name is: "+aName);	
      }

      catch(EOFException e)
      { // Since this is normal you could do nothing here
	System.out.println("End of File reached");              // Report end of file.
      }

      catch(IOException e)
      {
	System.err.println("IOException in reading Objects");
	System.err.println(e);
        System.exit(1);                                   	// End the program.
      }

      catch(ClassNotFoundException e)
      {
	System.err.println(e);
        System.exit(1);                                   	// End the program.
      }

      finally
      {
        if(out!=null)
	  out.close();                                    // Ensure the output file is closed.
        if(in!=null)
	  in.close();                                     // Ensure the input file is closed.
      }
    }

    // Handle IO exception from file close:
    catch(IOException e)
    {
      System.err.println(e);
    }
  }
}
