// *** For solution 3: an extra map (phonebook2) has been added to the existing one (now phonebook1)
// to accommodate the new PhoneNumber key. A getPerson() method has been added that returns
// a Person object for a given PhoneNumber key. The getEntry() is now in two 
//  overloaded versions with a Person or PhoneNumber argument. New entries will be stored 
// simultaneously in both phonebooks, which in their turn will be saved in the same .bin file.
// *** For solution 4, the addition consists of a method which compares a surname with
// the existing ones and displays the  entry(-ies) corresponding to matches. 

import java.util.*;
import java.io.*;

class PhoneBook implements Serializable
{

	public PhoneBook()
	{
		if(filename.exists())
		try
		{
			// The filename object is used in the constructor that reads the HashMap 
			// object from the file if it exists. If it doesn't exist it does nothing 
			// and the PhoneBook object will use the default empty HashMap object.

			ObjectInputStream in = new ObjectInputStream(new FileInputStream(filename));
			phonebook1 = (HashMap)in.readObject();	// *** Same object read in both maps.		
			phonebook2 = (HashMap)in.readObject();
			in.close();
		}
		catch(ClassNotFoundException e)
		{
			System.out.println(e);
		}
		catch(IOException e)
		{
			System.out.println(e);
		}
	}

	public void save()								// Stores the maps away.
	{
		try
		{
			System.out.println("Saving phone book");
			ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(filename));
			out.writeObject(phonebook1);			// *** Both maps stored in the same file.
			out.writeObject(phonebook2);
			System.out.println("Done");
			out.close();
		}
		catch(IOException e)
		{
			System.out.println(e);
		}
	}

	private File filename = new File("Phonebook.bin");

	// Method to store the same BookEntry object simultaneously in both phonebooks:
	public void addEntry(BookEntry entry)
	{
		phonebook1.put(entry.getPerson(), entry);
		phonebook2.put(entry.getNumber(), entry);		// ***.
	}

	// Methods to retrieve the entry using name or phone number keys:
	public BookEntry getEntry(Person key)
	{
		return (BookEntry)phonebook1.get(key);
	}

	public BookEntry getEntry(PhoneNumber key)			// ***.
	{
		return (BookEntry)phonebook2.get(key);
	}

	// Method to retrieve the number corresponding to the name: 
	public PhoneNumber getNumber(Person key)
	{
		return getEntry(key).getNumber();
	}

	// *** Method to retrieve the name corresponding to the number:
	public Person getPerson(PhoneNumber key)
	{
		return getEntry(key).getPerson();
	}

	// *** Method to retrieve the entry corresponding to the surname:
	public void getSurnameEntry(String theName)
	{
		Collection surnameEntry = phonebook1.values();	// *** Get the entries as a collection.
		Iterator iter = surnameEntry.iterator();		// *** Get the iterator for the collection.

		int count = 0;									// *** Count of successful matches.

		do
		{
			BookEntry entry = (BookEntry)iter.next();         // *** Get an individual entry.
			String surname = entry.getPerson().getSurname();  // *** Extract the surname from it.
			if(theName.equalsIgnoreCase(surname)==true)       // *** If it matches the surname:
			{
				System.out.println(entry);                      // *** display the corresponding entry;
				count++;                                        // *** increase the count.
			}
		}
		while(iter.hasNext());

		if (count == 0)                                     // *** If there are no successful matches:
		{
			System.out.println("Sorry, no matching entries.");	// *** say so.
		}
	}
	
	// Method to display all entries alphabetically (from map using name as key):
	public void listEntries()
	{
		LinkedList persons = new LinkedList(phonebook1.keySet());		// Get the keys as a list.
		Collections.sort(persons);					// Sort the keys in alphabetical order.
		Iterator iter = persons.iterator();			// Get the iterator for sorted keys.
		
		while(iter.hasNext())
			System.out.println(phonebook1.get((Person)iter.next()));  // Iterate over the keys.
	}

	private HashMap phonebook1 = new HashMap();	
	private HashMap phonebook2 = new HashMap();
}
