import java.util.*;

public class Supervisor implements Runnable
{
  Bank theBank;
  private String myName;
  private boolean busy;

  // Various arrays:
  private String[]  clerkNames = {"Allison", "Bob", "Claire", "Dave"};
  public  Clerk[]   clerks = new Clerk[clerkNames.length];
  private Thread[]  clerkThread = new Thread[clerks.length];

  // Constructor:
  public Supervisor(Bank theBank, String myName)
  {
    this.theBank = theBank;
    this.myName = myName;
    this.busy = true;
  }

  // getName method:
  public String getName()
  {
    return this.myName;
  }

  // The supervisor:
  synchronized public void run()
  {
    int transactionCount = 20;          // Number of debits and of credits.

    System.out.println(myName + " is ready to supervise!");

    // Create the clerks:
    for(int i = 0; i < clerks.length; i++)
      clerks[i] = new Clerk(theBank, clerkNames[i], this.myName);

    // Create the threads for the clerks as daemon, and start them off:
    for(int i = 0; i < clerks.length; i++) {
      clerkThread[i] = new Thread(clerks[i]);
      clerkThread[i].setDaemon(true);
      clerkThread[i].start();
    }

    // Create the accounts, and initialize total credits and debits:
    for(int i = 0; i < BankOperation.initialBalance.length; i++)
    {
      BankOperation.accounts[i] = new Account(i+1, BankOperation.initialBalance[i]); // Create accounts
      BankOperation.totalCredits[i] = BankOperation.totalDebits[i] = 0;
      BankOperation.nCredits[i] = BankOperation.nDebits[i] = 0;
    }

    // Generate transactions of each type and pass to the clerks:
    Random rand = new Random();                      // Random number generator.
    Transaction transaction;                         // Stores a transaction.
    int amount;                                      // Stores an amount of money.
    int select;                                      // Selects an account.

    for(int i = 1; i <= transactionCount; i++)
    {
      // Generate a credit or debit at random:
      if(rand.nextInt(2)==1)
      {
        // Generate a random account index for credit operation:
        select = rand.nextInt(BankOperation.accounts.length);
        amount = 50 + rand.nextInt(26);                // Generate amount of $50 to $75.
        transaction = new Transaction(BankOperation.accounts[select],   // Account.
                                          Transaction.CREDIT, 		// Credit transaction
                                          amount);            		//  of amount.
        BankOperation.totalCredits[select] += amount;                	// Keep total credit tally.
        BankOperation.nCredits[select]++;
      }

      else
      {
        // Generate a random account index for debit operation:
        select = rand.nextInt(BankOperation.accounts.length);
        amount = 30 + rand.nextInt(31);                // Generate amount of $30 to $60.
        transaction = new Transaction(BankOperation.accounts[select],   // Account.
                                  Transaction.DEBIT,  			// Debit transaction
                                  amount);            			//  of amount
        BankOperation.totalDebits[select] += amount;                 	// Keep total debit tally.
        BankOperation.nDebits[select]++;
      }

      // Pick a random clerk to take care of the transaction:
      clerks[rand.nextInt(clerks.length)].doTransaction(transaction);  	// Now do the transaction.
    }

    for(int i=0; i < clerks.length; i++)
      clerks[i].isBusy();

    busy = false;
    notifyAll();

    return;
  }

  // Busy check:
  synchronized public void isBusy()
  {
    while(busy)
      try
      {
        wait();
      }
      catch(InterruptedException e)
      {
        System.out.println(e);
      }
    return;
  }
}