// Class to define a dialog to choose a color
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.event.*;

class ColorSelectDialog extends JDialog
                 implements Constants,
                            ActionListener          // For buttons etc
{
  // Constructor
  public ColorSelectDialog(SketchFrame window)
  {
    // Call the base constructor to create a modal dialog
    super(window, "Color Selection", true);   

    this.window = window;                         // Save the window reference

    // Create the dialog button panel 
    JPanel buttonPane = new JPanel();             // Create the panel to hold buttons

    // Create and add the buttons to the buttonPane
    buttonPane.add(ok = createButton("OK"));              // Add the OK button
    buttonPane.add(cancel = createButton("Cancel"));      // Add the Cancel button
    getContentPane().add(buttonPane, BorderLayout.SOUTH); // Add pane to content pane

    // Code to create the data input panel...
    JPanel dataPane = new JPanel(new GridLayout(3,1));
    dataPane.setBorder(BorderFactory.createCompoundBorder(         // Create pane border
                       BorderFactory.createLineBorder(Color.black),
                       BorderFactory.createEmptyBorder(5, 5, 5, 5)));  

    dataPane.add(new JLabel("Select a color:"));

    colorCombo = new JComboBox(new String[] {"Red", "Yellow", "Green",
                                             "Blue", "Custom..."});
    colorCombo.addActionListener(this);
    dataPane.add(colorCombo);

    colorField = new JTextField("Sample", 10);
    colorField.setBackground(color);
    dataPane.add(colorField);

    getContentPane().add(dataPane, BorderLayout.CENTER);
    pack();
    setVisible(false);
  }

  JButton createButton(String label)
  {
    JButton button = new JButton(label);                // Create the button
    button.setPreferredSize(new Dimension(80,20));      // Set the size
    button.addActionListener(this);                     // Listener is the dialog
    return button;                                      // Return the button
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();                // Get the source of the event
    if(source == ok)                              // Is it the OK button?

    {
      window.setElementColor(color);              // Set the selected color
      setVisible(false);                          // Hide the dialog
    }
    else if(source == cancel)                     // If it is the Cancel button
      setVisible(false);                          // just hide the dialog
    else if(source == colorCombo)
    {
      String newColor = (String)colorCombo.getSelectedItem();
      if (newColor.equals("Custom..."))
      {
        Color thecolor = JColorChooser.showDialog(this, "Select Custom Color", color);
        if(thecolor != null)
        {
          color = thecolor;
        }
      }
      else if (newColor.equals("Red"))
      {
        color = Color.red;
      }
      else if (newColor.equals("Blue"))
      {
        color = Color.blue;
      }
      else if (newColor.equals("Green"))
      {
        color = Color.green;
      }
      else if (newColor.equals("Yellow"))
      {
        color = Color.yellow;
      }

      colorField.setBackground(color);
    }
  }

  private SketchFrame window;        // The application window
  private Color color = Color.red;                 // Currently selected font
  private JButton ok;                // OK button
  private JButton cancel;            // Cancel button
  private JComboBox colorCombo;
  private JTextField colorField;
}

