import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import javax.swing.*;

public class ImageDrawDemo extends JApplet
{
  // The init() method to initialize everything...
  public void init()
  {
    spritePanel = new SpritePanel();
    setGlassPane(spritePanel);
    spritePanel.setVisible(true);
    imagePanel = new ImagePanel(getSize());
    getContentPane().add(imagePanel);
  }
 
  // The start() method to start the animation...
  public void start()
  {
    spritePanel.startAnimating();
    imagePanel.startAnimating();
  }

  // The stop() method to stop the animation...
  public void stop()
  {
    spritePanel.stopAnimating();
    imagePanel.startAnimating();
  }

  // Parameter information for anyone that needs it
  public String[][] getParameterInfo()
  {
    String[][] info = {
             {"fadeTime" , "integer", "time to complete fade in seconds"},
             {"frameRate", "integer", "frames per second"               }
                      };
    return info;
  }


  // The ImagePanel class defining the panel displaying the image...
  class ImagePanel extends JPanel
  {  
    public ImagePanel(Dimension size)
    {
      // Get parameters - if any
      String fade = getParameter("fadeTime");
      if(fade != null)
        fadeTime = Integer.parseInt(fade);
      String fps = getParameter("frameRate");
      if(fps != null)
        frameRate = Integer.parseInt(fps);
      maxCount = frameRate*fadeTime;               // Count of steps to complete fade

      composite = AlphaComposite.SrcOver;

      ImageIcon icon = new ImageIcon("Images/wrox_logo.gif");
      image = icon.getImage();

      // Create a scaled image to fit within the size
      image = image.getScaledInstance(4*size.width/5, 4*size.height/5,
                                                                   Image.SCALE_SMOOTH);
      // Wait for scaled image to load
      MediaTracker tracker = new MediaTracker(this);
      tracker.addImage(image,0);                    // Image to track
      try
     {
      tracker.waitForID(0);
      }
      catch(InterruptedException e)
      {
      System.out.println(e);                       // Exception...
      System.exit(1);                              // ...so abandon ship!
      }
    }


    public void startAnimating()
    {
      timer = new java.util.Timer(true);                     // Timer to run clock        
      count = maxCount;                                      // Set repaint counter
      alphaStep = 1.0f/count;
      long frameInterval = ONE_SECOND/frameRate;

      // Use fixed-delay execution to get smooth fade
      timer.schedule(new java.util.TimerTask()
                   {
                     public void run()
                     {
                       imagePanel.repaint();              // Repaint the image

                       // Update alpha composite for next frame
                       if(count ==maxCount)
                         countDelta = -1;
                       else if(count == 0)
                         countDelta = 1;
                       count += countDelta;
                       composite = AlphaComposite.getInstance(
                                               AlphaComposite.SRC_OVER,count*alphaStep);
                     }
                      
                     int countDelta = -1;      // Anonymous class member - count incr.
                   },
                   0,                                     // ...starting now
                   frameInterval);                        // Repaint interval
    }

    public void stopAnimating()
    {
      timer.cancel();
    }


    public void paint(Graphics g)
    {
      Graphics2D g2D = (Graphics2D)g;
      Dimension size = getSize();                    // Get panel size
      g2D.setPaint(Color.lightGray);                 // Background color
      g2D.fillRect(0,0,size.width,size.height);      // fill the panel
      g2D.setComposite(composite);                   // Set current alpha

      // Scale and draw image
      g2D.drawImage(image,                           // Image to be drawn
                    size.width/10,size.height/10,    // Image position inset
                    null);
    }

    Image image;                 // The image
    int imageWidth;              // and its width
    int imageHeight;             // and height

    final int ONE_SECOND = 1000;  // One second in milliseconds
    int frameRate = 20;           // Default fade change frequency frames per sec
    int fadeTime = 3;             // Default time to fade completely in seconds
    int count;                    // Repaint cycle counter
    int maxCount;
    AlphaComposite composite;     // Alpha value for the image
    float alphaStep;              // Alpha increment for fade step
    java.util.Timer timer;        // Timer to control fading  

  }



  // The SpritePanel class defining the panel displaying the sprite...
  class SpritePanel extends JPanel
  {  
    public SpritePanel()
    {
      sprite = createSprite(spriteSize);
      setOpaque(false);
    }


    // The startAnimating() method to start the animation...
    public void startAnimating()
    {
      timer = new java.util.Timer(true); 

      // Do necessary initialization...
      totalAngle = 0;                            // Position around the circle
      spriteAngle = 0;                           // Sprite orientation

      // Use fixed-delay execution to get smooth animation
      timer.schedule(new java.util.TimerTask()
      {
        public void run()
        {
          // Painting for animation + necessary updating
          spritePanel.repaint();               // Repaint the image
          double secondsPerRotation = 15;     // Time for one complete revolution
          double angle = 2.*interval*Math.PI/(1000.0*secondsPerRotation);
          double spriteAngleIncrement = -4*angle;
          totalAngle += angle;                // Increment the total angle
          spriteAngle += spriteAngleIncrement;// and the sprite angle
        }
      },
      0,                                    // ...starting now
      interval);                            // Repaint interval
    }


    // The stopAnimating() method to stop the animation...
    public void stopAnimating()
    {
      timer.cancel();                        // Stop the timer
    }


    public void paint(Graphics g)
    {
      Dimension size = getSize();                        // Get panel dimensions
      Graphics2D g2D = (Graphics2D)g;
       
      // Now fill the panel background
      g2D.setPaint(CLEAR);
      g2D.fillRect(0,0,size.width,size.height);
      g2D.setPaint(Color.darkGray);

      g2D.translate(size.width/2, size.height/2);       // Move origin to panel center

      g2D.rotate(totalAngle);            // Rotate to the angle for the sprite position

      // Translate and rotate to sprite position - then translate back
      g2D.rotate(spriteAngle, 3*spriteSize/2, 0);

      g2D.translate(spriteSize, -spriteSize/2);         // Translate to sprite top left

      g2D.drawImage(sprite ,null, 0, 0);                // Draw the sprite
    }


    void drawArm(Graphics2D g2D, int i)
    {
      AffineTransform at = g2D.getTransform();        // Save current transform
      g2D.setPaint(Color.darkGray);
      Stroke stroke = g2D.getStroke();                // Save current stroke
      g2D.setStroke(new BasicStroke(3.0f));           // Set stroke as wide line
      g2D.draw(line);                                 // Draw the line
      g2D.setStroke(stroke);                          // Restore old stroke

      g2D.translate(line.getX2(), -spriteSize/20);    // Translate to circle position
      g2D.setPaint(colors[i%colors.length]);          // Set the fill color
      g2D.fill(circle);
      g2D.setTransform(at);                           // Restore original transform()
    }


    BufferedImage createSprite(int spriteSize)
    {
      // Create image with RGB and alpha channel
      BufferedImage sprite = new BufferedImage(spriteSize, spriteSize,
                                               BufferedImage.TYPE_INT_ARGB);

      Graphics2D g2D = sprite.createGraphics();        // Context for buffered image

      // Set best alpha interpolation quality
      g2D.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION,
                           RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);

      // Clear image with transparent alpha by drawing a rectangle
      g2D.setComposite(AlphaComposite.getInstance(AlphaComposite.CLEAR, 0.0f));
      Rectangle2D.Double rect = new Rectangle2D.Double(0,0,spriteSize,spriteSize); 
      g2D.fill(rect);
	    
      g2D.setComposite(AlphaComposite.SrcOver);
      line = new Line2D.Double(spriteSize/20.0,0,0.3*spriteSize,0);
      circle = new Ellipse2D.Double(0,0,spriteSize/10.0, spriteSize/10.0);
	
      // Since sprite is symmetric, move origin to the center
      g2D.translate(spriteSize/2, spriteSize/2);

      int armCount = 8;                           // Number of arms in the sprite
      for(int i = 0; i < armCount; i++)
      {
        g2D.rotate(2*Math.PI/armCount);           // Rotate by pi/4
        drawArm(g2D, i);                          // Draw the arm
      }
      g2D.setPaint(Color.lightGray);                   // Set the fill color
      g2D.translate(-spriteSize/20,-spriteSize/20);    // Move origin to circle top left
      g2D.fill(circle);                                // Fill the circle
      g2D.dispose();                                   // Dispose of the context
      return sprite;                                   // Return the finished image
    }


    double totalAngle;                       // Current angular position of sprite
    double spriteAngle;                      // Rotation angle of sprite about its center

    BufferedImage sprite;                    // Stores reference to the sprite
    Ellipse2D.Double circle;                 // A circle - part of the sprite
    Line2D.Double line;                      // A line - part of the sprite

    // Colors used in sprite
    Color[] colors = {Color.red , Color.yellow, Color.green  , Color.blue,
    Color.cyan, Color.pink  , Color.magenta, Color.orange};

    java.util.Timer timer;                   // Timer for the animation
    long interval = 50;

    final Color CLEAR = new Color(0,0,0,0);
  }


  ImagePanel imagePanel;                   // Panel to display image
  SpritePanel spritePanel;                 // Panel to display sprite
  int spriteSize = 100;                    // Diameter of the sprite

}