import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import javax.swing.*;

public class ImageDrawDemo extends JApplet
{
  // The init() method to initialize everything...
  public void init()
  {
    imagePanel = new ImagePanel();
    getContentPane().add(imagePanel);
  }
 
  // The start() method to start the animation...
  public void start()
  {
    imagePanel.startAnimating();
  }

  // The stop() method to stop the animation...
  public void stop()
  {
    imagePanel.stopAnimating();
  }

  // The ImagePanel class defining the panel displaying the animation...
  class ImagePanel extends JPanel
  {  
    public ImagePanel()
    {
      ImageIcon icon = new ImageIcon("Images\\wrox_logo.gif");
      image = icon.getImage();
      sprite = createSprite(spriteSize);
      timer = new java.util.Timer(true); 
    }

    public void startAnimating()
    {
      // Check all images are painted
      calculateAndPaint();

      // Use fixed-delay execution to get smooth animation
      timer.schedule(new java.util.TimerTask()
        {
          public void run()
          {
            // Paint next image
            imagePanel.repaint();

            // Update to do next image
            currentImage = (currentImage+1)%framesPerRotation;
          }
        },
        0,                                    // ...starting now
        interval);                            // Repaint interval
    }


    // Draw all the images and store them for later replay
    public void calculateAndPaint()
    {
      // Do necessary initialization...
      framesPerRotation = (int)((1000 * (int)secondsPerRotation) / interval);
      totalAngle = 0;                            // Position around the circle
      spriteAngle = 0;                           // Sprite orientation
      double angleIncrement = 2.*interval*Math.PI/(1000.0*secondsPerRotation);
      double spriteAngleIncrement = -4*angleIncrement;

      Dimension size = getSize();                        // Get panel dimensions

      images = new BufferedImage[framesPerRotation];

      for (int i=0; i<framesPerRotation; i++)
      {
        images[i] = new BufferedImage(size.width, size.height,
                                      BufferedImage.TYPE_INT_ARGB);

        // Paint this image

        // Get Graphics object for the image.
        Graphics2D g2D = images[i].createGraphics();        // Context for buffered image

        // Now fill the panel background
        g2D.setPaint(Color.gray);
        Rectangle2D.Double rect = new Rectangle2D.Double(0, 0, size.width, size.height);
        g2D.fill(rect);

        g2D.translate(size.width/2, size.height/2);       // Move origin to panel center

        g2D.drawImage(image,                                     // Draw the logo
                      -image.getWidth(ImageDrawDemo.this)/2,     // Top left x
                      -image.getWidth(ImageDrawDemo.this)/2,     // Top left y
                      ImageDrawDemo.this);

        g2D.rotate(totalAngle);      // Rotate to the angle for the sprite position

        // Translate and rotate to sprite position - then translate back
        g2D.rotate(spriteAngle, 3*spriteSize/2, 0);

        g2D.translate(spriteSize, -spriteSize/2);         // Translate to sprite top left

        g2D.drawImage(sprite, null, 0, 0);                // Draw the sprite


        // Do the next one 

        totalAngle += angleIncrement;         // Increment the total angle
        spriteAngle += spriteAngleIncrement;  // and the sprite angle
      }
    }

    public void stopAnimating()
    {
      timer.cancel();                        // Stop the timer
    }

    public void paint(Graphics g)
    {
      Graphics2D g2D = (Graphics2D)g;
      g2D.drawImage(images[currentImage], 0, 0, ImageDrawDemo.this);
    }

    void drawArm(Graphics2D g2D, int i)
    {
      AffineTransform at = g2D.getTransform();        // Save current transform
      g2D.setPaint(Color.darkGray);
      Stroke stroke = g2D.getStroke();                // Save current stroke
      g2D.setStroke(new BasicStroke(3.0f));           // Set stroke as wide line
      g2D.draw(line);                                 // Draw the line
      g2D.setStroke(stroke);                          // Restore old stroke

      g2D.translate(line.getX2(), -spriteSize/20);    // Translate to circle position
      g2D.setPaint(colors[i%colors.length]);          // Set the fill color
      g2D.fill(circle);
      g2D.setTransform(at);                           // Restore original transform()
    }

    BufferedImage createSprite(int spriteSize)
    {
      // Create image with RGB and alpha channel
      BufferedImage sprite = new BufferedImage(spriteSize, spriteSize,
                                               BufferedImage.TYPE_INT_ARGB);

      Graphics2D g2D = sprite.createGraphics();        // Context for buffered image

      // Set best alpha interpolation quality
      g2D.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION,
                           RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);

      // Clear image with transparent alpha by drawing a rectangle
      g2D.setComposite(AlphaComposite.getInstance(AlphaComposite.CLEAR, 0.0f));
      Rectangle2D.Double rect = new Rectangle2D.Double(0,0,spriteSize,spriteSize); 
      g2D.fill(rect);
	    
      g2D.setComposite(AlphaComposite.SrcOver);
      line = new Line2D.Double(spriteSize/20.0,0,0.3*spriteSize,0);
      circle = new Ellipse2D.Double(0,0,spriteSize/10.0, spriteSize/10.0);
	
      // Since sprite is symmetric, move origin to the center
      g2D.translate(spriteSize/2, spriteSize/2);

      int armCount = 8;                           // Number of arms in the sprite
      for(int i = 0; i < armCount; i++)
      {
        g2D.rotate(2*Math.PI/armCount);           // Rotate by pi/4
        drawArm(g2D, i);                          // Draw the arm
      }
      g2D.setPaint(Color.lightGray);                   // Set the fill color
      g2D.translate(-spriteSize/20,-spriteSize/20);    // Move origin to circle top left
      g2D.fill(circle);                                // Fill the circle
      g2D.dispose();                                   // Dispose of the context
      return sprite;                                   // Return the finished image
    }

    int spriteSize = 100;            // Diameter of the sprite
    Ellipse2D.Double circle;         // A circle - part of the sprite
    Line2D.Double line;              // A line - part of the s
    BufferedImage sprite;            // Stores reference to the sprite
    BufferedImage[] images;          // The images which will be displayed in the ImagePanel
    java.util.Timer timer;
    long interval = 50;              // Milliseconds between repaints
    double secondsPerRotation = 5;   // Time for one complete revolution
    int framesPerRotation;
    Image image;                     // The logo image
    double totalAngle;               // Current angular position of sprite
    double spriteAngle;              // Rotation angle of sprite about its center
    int currentImage = 0;            // Image now being shown


    // Colors used in sprite
    Color[] colors = {Color.red , Color.yellow, Color.green  , Color.blue,
    Color.cyan, Color.pink  , Color.magenta, Color.orange};
  }

  ImagePanel imagePanel;                   // Panel to display animation

}