import java.awt.*;
import java.awt.event.*;                 // For event classes
import javax.swing.*;                    // For Swing components
import javax.swing.table.*;              // For the table classes
import java.sql.*;                       // For JDBC classes

public class InteractiveSQL extends JFrame implements ActionListener
{
  public static void main(String[] args)
  { // Create the application object
    // Set default values for the command line args
    String user     = "guest";
    String password = "guest";
    String url      = "jdbc:odbc:technical_library";
    String driver   = "sun.jdbc.odbc.JdbcOdbcDriver";

    // Up to 4 arguments in the sequence database url,driver url, user ID, password
    switch(args.length)
    {
      case 4:                                 // Start here for four arguments
        password = args[3];
        // Fall through to the next case
      case 3:                                 // Start here for three arguments
        user = args[2];
        // Fall through to the next case
      case 2:                                 // Start here for two arguments
        driver = args[1];
        // Fall through to the next case
      case 1:                                 // Start here for one argument
        url = args[0];
    }

    InteractiveSQL theApp = new InteractiveSQL(driver, url, user, password);

  }

  public InteractiveSQL(String driver, String url, String user, String password)
  {
    super("InteractiveSQL");                           // Call base constructor
    setBounds(0, 0, 400, 300);                         // Set window bounds
    setDefaultCloseOperation(DISPOSE_ON_CLOSE);        // Close window operation
    addWindowListener(new WindowHandler());            // Listener for window close

    // Add the input for SQL statements at the top
    command.setToolTipText("Key SQL commmand and press Enter");
    command.setEditable(true);
    command.addActionListener(this);
    getContentPane().add(command, BorderLayout.NORTH);

    // Add the status reporting area at the bottom
    status.setLineWrap(true);
    status.setWrapStyleWord(true);
    getContentPane().add(status, BorderLayout.SOUTH);

    // Create the menubar from the menu items
    JMenu fileMenu = new JMenu("File");                // Create File menu
    fileMenu.setMnemonic('F');                         // Create shortcut
    clearQueryItem.addActionListener(this);
    changeDatabaseItem.addActionListener(this);
    exitItem.addActionListener(this);
    fileMenu.add(clearQueryItem);                      // Add clear query item
    fileMenu.add(changeDatabaseItem);
    fileMenu.add(exitItem);                            // Add exit item
    menuBar.add(fileMenu);                             // Add menu to the menubar
    setJMenuBar(menuBar);                              // Add menubar to the window

    databaseDialog = new DatabaseDialog(this, driver, url, user, password);

    // Establish a database connection and set up the table
    try
    {
      connectToDatabase(driver, url, user, password);
    }
    catch(ClassNotFoundException cnfe)
    {
      System.err.println(cnfe);                    // Driver not found
    }
    catch(SQLException sqle)
    {
      System.err.println(sqle);                    // error connection to database
    }

    model = new ResultsModel();                  // Create a table model
    JTable table = new JTable(model);            // Create a table from the model
    table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);   // Use scrollbars
  
    resultsPane = new JScrollPane(table);        // Create scrollpane for table
    getContentPane().add(resultsPane, BorderLayout.CENTER);

    pack();
    setVisible(true);
  }

  public void connectToDatabase(String driverClass, String database, String username, String password)
    throws ClassNotFoundException, SQLException
  {
    Class.forName(driverClass);                  // Load the driver
    connection = DriverManager.getConnection(database,
                                             username,
                                             password);
    statement = connection.createStatement();
  }

  
  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if(source == command)                       // Enter key for text field input
    {
      if (!(command.getSelectedItem().equals("")))
      {

        if (command.getSelectedIndex() == -1)
        {
          command.insertItemAt(command.getSelectedItem(), 0);
          if (command.getItemCount() == 11)
          {
            command.removeItemAt(10);
          }
        }      

      executeSQL((String)command.getSelectedItem());
      }
    }

    else if(source == changeDatabaseItem)
    {
      Rectangle bounds = getBounds();
      databaseDialog.setLocation(bounds.x + bounds.width/3, bounds.y + bounds.height/3);
      databaseDialog.setVisible(true);
    }

    else if(source == clearQueryItem)           // Clear query menu item
      command.setSelectedItem("");                      // Clear SQL entry

    else if(source == exitItem)                 // Exit menu item
    {
      dispose();                                 // Release the window resources
      System.exit(0);                            // End the application
    }    
  }

  public void executeSQL(String query)
  {
    if(query == null)                      // If there's nothing we are done
      return;
    try
    {
      model.setResultSet(statement.executeQuery(query));
      status.setText("Resultset has " + model.getRowCount() + " rows.");
    }
    catch (SQLException sqle)
    {
      status.setText(sqle.getMessage());        // Display error message
    }
  }

  class WindowHandler extends WindowAdapter
  {
    // Handler for window closing event
    public void windowClosing(WindowEvent e)
    {
      dispose();                                 // Release the window resources
      System.exit(0);                            // End the application
    }
  }

  JPanel topPanel = new JPanel();              // Panel for command string and database name
  JLabel commandLabel = new JLabel("SQL command:");
  JComboBox command = new JComboBox();       // Input area for SQL
  JTextArea status = new JTextArea(3,1);       // Output area for status and errors
  JScrollPane resultsPane;

  JMenuBar menuBar = new JMenuBar();                        // The menu bar
  JMenuItem clearQueryItem = new JMenuItem("Clear query");  // Clear SQL item
  JMenuItem changeDatabaseItem = new JMenuItem("Change database");
  JMenuItem exitItem = new JMenuItem("Exit");               // Exit item

  DatabaseDialog databaseDialog;

  Connection connection;                       // Connection to the database
  Statement statement;                         // Statement object for queries
  ResultsModel model;                          // Table model for resultset
}
