import java.sql.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import javax.swing.event.*;

class DatabaseBrowse extends JFrame
                      implements ActionListener, TreeSelectionListener
{
  public static void main(String[] args)
  {
     DatabaseBrowse theApp = new DatabaseBrowse();    // Create application object
  }
  public DatabaseBrowse()
  {
    super("Database Browser");
    
    setBounds(0, 0, 400, 300);
    setDefaultCloseOperation(DISPOSE_ON_CLOSE);
    addWindowListener(new WindowHandler());
    // Create labels for input fields
    JLabel dbURLLabel  = new JLabel( "Database URL: ");
    JLabel userIDLabel = new JLabel( "User ID: ", JLabel.RIGHT);
    userIDLabel.setPreferredSize(dbURLLabel.getPreferredSize()); // Set same size
    JLabel passwordLabel = new JLabel("Password: ");
    // Box for database URL input
    Box dbPane = Box.createHorizontalBox();
    dbPane.add(dbURLLabel);
    dbPane.add(database);

    // Box for user ID and password input fields
    Box loginPane = Box.createHorizontalBox();
    loginPane.add(userIDLabel);
    loginPane.add(userIDInput);
    loginPane.add(passwordLabel);
    loginPane.add(passwordInput);

    Box queryPane1 = Box.createHorizontalBox();
    JLabel selectLabel = new JLabel("SELECT");
    JLabel fromLabel = new JLabel("FROM");
    tableField = new JTextField(15);
    tableField.setEditable(false);
    columnsField = new JTextField(15);
    columnsField.setEditable(false);

    queryPane1.add(selectLabel);
    queryPane1.add(columnsField);
    queryPane1.add(fromLabel);
    queryPane1.add(tableField);
    
    Box queryPane2 = Box.createHorizontalBox();

    JLabel whereLabel = new JLabel("WHERE");
    whereLabel.setPreferredSize(selectLabel.getPreferredSize());
    JLabel orderLabel = new JLabel("ORDER BY");
    performQueryButton.addActionListener(this);

    queryPane2.add(whereLabel);
    queryPane2.add(whereInput);
    queryPane2.add(orderLabel);
    queryPane2.add(orderInput);

    getRootPane().setDefaultButton(performQueryButton);

    Box inputPane = Box.createVerticalBox();
    inputPane.add(dbPane);
    inputPane.add(loginPane);
    inputPane.add(Box.createVerticalStrut(5));
    inputPane.add(queryPane1);
    inputPane.add(queryPane2);
    inputPane.add(performQueryButton);

    getContentPane().add(inputPane, BorderLayout.NORTH);

    // Add message area
    status.setText("Enter a database URL and/or press Enter");
    status.setEditable(false);                    // No user input
    status.setLineWrap(true);                     // Lines wrap
    status.setWrapStyleWord(true);                // on word boundaries
    status.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
    getContentPane().add(status, BorderLayout.SOUTH);

    // Create tree to go in left split pane
    dbNode = new DefaultMutableTreeNode("No database");
    dbTreeModel = new DefaultTreeModel(dbNode);
    dbTree = new JTree(dbTreeModel);
    treePane = new JScrollPane(dbTree);
    treePane.setBorder(BorderFactory.createLineBorder(Color.darkGray));

    // Create table to go in right split pane
    tableModel = new ResultsModel();
    JTable table = new JTable(tableModel);
    table.setAutoCreateColumnsFromModel(true);
    table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF); 
    tablePane = new JScrollPane(table);
    tablePane.setBorder(BorderFactory.createLineBorder(Color.darkGray));

    JSplitPane splitpane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                                          true,             // Continuous relayout
                                          treePane,         // Left pane content
                                          tablePane);       // Right pane content
    getContentPane().add(splitpane, BorderLayout.CENTER);
    splitpane.setDividerLocation(150);                 // Left pane 150 pixels wide

    // Add event listeners
    database.addActionListener(this);
    userIDInput.addActionListener(this);
    passwordInput.addActionListener(this);
    dbTree.addTreeSelectionListener(this);

    pack();

    setVisible(true);                            // Set window visible
    show();                                      // Display the window
    database.requestFocus();                     // Focus to the url input field

    // Attempt to load all drivers
    for (int i = 0 ; i < drivers.length ; i++)
      try
      {
        Class.forName(drivers[i]);
      }
      catch(ClassNotFoundException cnfe)
      {
        System.err.println(cnfe);
        status.setText("Driver load failed: " + cnfe.getMessage());
      }

  }


  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();            // Get source of the event
    if(source == database    ||               // If its URL input,
       source == userIDInput ||               // or userID input,
       source == passwordInput)               // or password input,
    {                                         // we will try for a connection
      url = database.getText();               // Get database URL
      userID = userIDInput.getText();               // Get user ID

      char[] pw = passwordInput.getPassword();      // Get password
      if(pw != null)
        password = new String(pw);

     if(url == null || url.length()==0)
      {
        status.setText("Please specify a database URL ");
        return;
      }
      openConnection();
      password = null;                             // For security
    }
    else if (source == performQueryButton)
    {
      if ((columnsToView != null) && (tableToView != null))
      {

        String where = whereInput.getText();
        String order = orderInput.getText();

        String queryString;

        if ((where == null) || (where.equals("")))
        {
          if ((order == null) || (order.equals("")))
          {
            queryString = "SELECT " + columnsToView + " FROM " + tableToView;
          }
          else
          {
            queryString = "SELECT " + columnsToView + " FROM " + tableToView +
                            " ORDER BY " + order;
          }
        }
        else
        {
          if ((order == null) || (order.equals("")))
          {
            queryString = "SELECT " + columnsToView + " FROM " + tableToView +
                            " WHERE " + where;
          }
          else
          {
            queryString = "SELECT " + columnsToView + " FROM " + tableToView +
                            " WHERE " + where + " ORDER BY " + order;
          }
        }
        try
        {
          // Display the columns and change the scroll pane border
          tableModel.setResultSet(statement.executeQuery(queryString));
          tablePane.setBorder(BorderFactory.createTitledBorder(
                             BorderFactory.createLineBorder(Color.darkGray),
                             tableToView,
                             TitledBorder.CENTER,
                             TitledBorder.DEFAULT_POSITION));
          status.setText("Query performed successfully");
        }
        catch(SQLException sqle)
        {
          String message = "SQL Error performing SELECT\n" + sqle.getMessage();
          System.err.println(message);
          status.setText(message);
        }
      }
    }
  }


  public void openConnection()
  {
    try
    {

      // Code to close the old connection as before....

      // Now open the new connection
      connection = DriverManager.getConnection(url, userID, password);
      status.setText("Database connection established");
      statement = connection.createStatement();       // Create statement for query
 

      dbNode = new DefaultMutableTreeNode(url);       // Root node is URL
      dbTreeModel.setRoot(dbNode);                    // Set root in model
      setupTree(connection.getMetaData());            // Set up tree with metadata

      treePane.setBorder(BorderFactory.createTitledBorder(
                         BorderFactory.createLineBorder(Color.darkGray),
                         url,
                         TitledBorder.CENTER,
                         TitledBorder.DEFAULT_POSITION));
      dbTree.setRootVisible(false);                   // Now show the root node
      dbTreeModel.reload();                           // Get the tree redisplayed
    }
    catch(SQLException sqle)
    {
      status.setText(sqle.getMessage());              // Display first message
      do                                              // loop through exceptions
      {
        System.err.println("Exception occurred:\nMessage: " + sqle.getMessage());
        System.err.println("SQL state: " + sqle.getSQLState());
        System.err.println("Vendor code: " + sqle.getErrorCode() +
                           "\n----------------");
      }
      while((sqle = sqle.getNextException()) != null);
    }
  }


  private void setupTree(DatabaseMetaData metadata) throws SQLException
  {
    String[] tableTypes = { "TABLE"};                  // We want only tables
    ResultSet tables = metadata.getTables(             // Get the tables info
                                          null,
                                          null,
                                          null,
                                          tableTypes);

      String tableName;                          // Stores a table name
      DefaultMutableTreeNode tableNode;          // Stores a tree node for a table
      while(tables.next())                             // For each table
      {
        tableName = tables.getString("TABLE_NAME");    // get the table name
        tableNode = new DefaultMutableTreeNode(tableName);
        dbNode.add(tableNode);                         // Add the node to the tree

        // Get all the columns for the current table
        ResultSet columnNames = metadata.getColumns(null, null, tableName, null);

        // Add nodes for the columns as children of the table node
        while(columnNames.next())
          tableNode.add(new 
                    DefaultMutableTreeNode(columnNames.getString("COLUMN_NAME")));
      }
  }


  public void valueChanged(TreeSelectionEvent e)
  {
    TreePath[] paths = dbTree.getSelectionPaths();
    if(paths == null)
      return;

    boolean tableSelected = false;             // Set true if a table is selected
    String column;                             // Stores a column name from a path
    String table;                              // Stores a table name from a path
    String columnsParam = null;                // Column names in SQL SELECT
    String tableParam = null;                  // Table name in SQL SELECT
    for(int  j = 0; j < paths.length ; j++)
    {
      switch(paths[j].getPathCount())
      {
        case 2:                                // We have a table selected
          tableParam = (String)
                          (((DefaultMutableTreeNode)
                                 (paths[j].getPathComponent(1))).getUserObject());
          columnsParam = "*";               // Select all columns
          tableSelected = true;             // Set flag for a table selected
          break;

        case 3:                               // Column selected
          table = (String)
                      (((DefaultMutableTreeNode)
                                  (paths[j].getPathComponent(1))).getUserObject());
          if(tableParam == null)
            tableParam = table;

          else if(tableParam != table)
            break;
          column = (String)
                       (((DefaultMutableTreeNode)
                                  (paths[j].getPathComponent(2))).getUserObject());
          if(columnsParam == null)           // If no previous columns
            columnsParam = column;           // add the column
          else                               // otherwise
            columnsParam += "," + column;    // we need a comma too
          break;
      }
      if(tableSelected)                      // If a table was selected
        break;                               // we are done
    }

    tableToView = tableParam;
    columnsToView = columnsParam;

    tableField.setText(tableParam);
    columnsField.setText(columnsParam);
    status.setText("Click 'Perform query' to execute query");
  }


  private String userID = "guest";
  private String password = "guest";
  private String url = "jdbc:odbc:technical_library";

  private JTextField database = new JTextField(url);
  private JTextField userIDInput = new JTextField(userID);
  private JPasswordField passwordInput = new JPasswordField(password);
  private JTextArea status = new JTextArea(3,30);
  private JTextField whereInput = new JTextField(15);
  private JTextField orderInput = new JTextField(15);

  private JTextField tableField, columnsField;

  private JButton performQueryButton = new JButton("Perform query");

  private DefaultMutableTreeNode dbNode;      // Root node for the database tree
  private DefaultTreeModel dbTreeModel;       // Model for the database metadata
  private JTree dbTree;                       // Tree to display the metadata
  private JScrollPane treePane;               // Scroll pane holding the tree

  ResultsModel tableModel;                    // Model for table
  private JTable table;                       // Table holding table data
  JScrollPane tablePane;                      // Scroll pane holding the table

  Connection connection;
  Statement statement;

  String tableToView;
  String columnsToView;

  private String[] drivers = {
                              "sun.jdbc.odbc.JdbcOdbcDriver"       // ODBC bridge
                             };


  // Inner class defining handler for window events
  class WindowHandler extends WindowAdapter
  {
    // Handler for window closing event
    public void windowClosing(WindowEvent e)
    {
      dispose();                                 // Release the window resources
      System.exit(0);                            // End the application
    }
  }
}
