import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import java.net.*;
import java.awt.geom.*;                 // For AffineTransform

public class TimedWhirlingLogo extends JApplet
{
  // This method is called when the applet is loaded
  public void init()
  {
    tracker = new MediaTracker(this);
    Image image = null;
    try
    {
      // Image from a file specified by a URL
      image = getImage(new URL(getCodeBase(),"Images/wrox_logo.gif")); 
    }
    catch(MalformedURLException e)
    {
      System.out.println("Failed to create URL:\n" + e);
    }
    tracker.addImage(image,0);                    // Load image
    try
    {
      tracker.waitForAll();                       // Wait for image to load
      if(tracker.isErrorAny())                    // If there is an error
         return;                                  // give up

      Dimension size = getSize();                   // Get applet size
      imageWidth = image.getWidth(this);            // Get image width
      imageHeight = image.getHeight(this);          // and its height

      // Calculate scale factor so diagonal of image fits width and height
      double diagonal = Math.sqrt(imageWidth*imageWidth + imageHeight*imageHeight);
      double scaleFactor = Math.min(size.width/diagonal, size.height/diagonal);

      // Create a transform to translate and scale the image
      at.setToTranslation((size.width-imageWidth*scaleFactor)/2,
                          (size.height-imageHeight*scaleFactor)/2);
      at.scale(scaleFactor,scaleFactor);

      imagePanel = new ImagePanel(image);         // Create panel showing the image
      getContentPane().add(imagePanel);           // Add the panel to the content pane
    }
    catch(InterruptedException e)
    {
      System.out.println(e);
    }
  }

  // This method is called when the browser starts the applet
  public void start()
  {
    if(tracker.isErrorAny())                   // If any image errors
      return;                                  // don't create the thread

    timer = new java.util.Timer(true);
    timer.schedule(new java.util.TimerTask()
    {
      public void run()
      {
        imagePanel.repaint();       // Repaint the image
        angle = 2.0*Math.PI*stepCount++/ STEPS_PER_ROTATION;
        stepCount = ++stepCount%STEPS_PER_ROTATION;
      }
    },
    0, INTERVAL);
  }

  // This method is called when the browser wants to stop the applet 
  //  - when is it not visible for example
  public void stop()
  {
    timer.cancel();
  }

  // Class representing a panel displaying an image
  class ImagePanel extends JPanel
  {
    public ImagePanel(Image image)
    {
      this.image = image;
    }

    public void paint(Graphics g)
    {
      Graphics2D g2D = (Graphics2D)g;
      g2D.transform(at);                                   // Apply scale & translate
     
      g2D.setPaint(Color.lightGray);
      g2D.fillRect(0, 0, imageWidth, imageHeight);

      g2D.rotate(angle, imageWidth/2.0, imageHeight/2.0);  // Rotate about center

      // draw scaled imaged with background
      g2D.drawImage(image, 0, 0, this);
    }
    Image image;                                           // The image
  }
  
  java.util.Timer timer;          // Animation timer
  MediaTracker tracker;                                // Tracks image loading
  ImagePanel imagePanel;
  AffineTransform at = new AffineTransform();   
  int imageWidth, imageHeight;                         // Image dimensions
  double angle;                                        // Rotation angle 
  final int INTERVAL = 50;                             // Time interval msec
  final int ROTATION_TIME = 2000;                      // Complete rotation time msec
  final int STEPS_PER_ROTATION = ROTATION_TIME/ INTERVAL;
  int stepCount;                                       // Total number of steps
}