import java.awt.*;
import javax.swing.*;
import java.awt.geom.*;
import java.util.*;

public class NewClock extends JApplet
{
  // This method is called when the applet is loaded
  public void init()
  {
    Dimension size = getSize();             // Get the applet size

    // Create the clockface to fit within the applet
    int clockDiameter = Math.min(size.width, size.height)*9/10;
    clock = new ClockFace(clockDiameter);
    getContentPane().add(clock);            // Add clockface to content pane      

    hands = new Hands(clockDiameter);       // Create the hands panel
    setGlassPane(hands);                    // Make the hands the glass pane                   
    hands.setVisible(true);                 // Set glass pane visible
  }

  // This method is called when the browser starts the applet
  public void start()
  {
    Calendar now = Calendar.getInstance();       // Calendar for this instant

    // Get current seconds, minutes, and hours
    seconds = now.get(now.SECOND);
    minutes = now.get(now.MINUTE);
    hours = now.get(now.HOUR);

    timer = new java.util.Timer(true);               // Timer to run clock        

    // Use fixed-rate execution to maintain time
    timer.scheduleAtFixedRate(new TimerTask()
    {
      public void run()
      {
        increment();                 // Increment the time
        hands.repaint();             // and repaint the hands
      }
    },
    0,                               // ...starting now
    ONE_SECOND);                     // and at one second intervals
  }

  // This method is called when the browser wants to stop the applet
  public void stop()
  {
    timer.cancel();                   // Cancel the clock timer
  }

  // Class defining the hands on the clock
  class Hands extends JPanel
  {
    public Hands(int clockDiameter)
    {
      center = new Ellipse2D.Double(-3,-3,6,6);                    // Central boss
      hourHand = new Line2D.Double(0,6,0,-clockDiameter*0.25);
      minuteHand = new Line2D.Double(0,8,0,-clockDiameter*0.3);
      secondHand = new Line2D.Double(0,14,0,-clockDiameter*0.35);
      setOpaque(false);                                            // Set transparent
    }
    
    // Paint the hands
    public void paint(Graphics g)
    {
      // Get hand angles for the current time
      double secondAngle = seconds*TWO_PI/60;
      double minuteAngle = (secondAngle+minutes*TWO_PI)/60;
      double hourAngle   = (minuteAngle+hours*TWO_PI)/12;

      Dimension size = getSize();
      Graphics2D g2D = (Graphics2D)g;
      g2D.setPaint(CLEAR);                       // Transparent color
      g2D.fillRect(0,0,size.width,size.height);  // Fill to erase

      g2D.setPaint(Color.darkGray);              // Hands color
      g2D.translate(size.width/2, size.height/2);// Origin to center
      AffineTransform transform = g2D.getTransform();  // Save this xform

      // Draw hour hand
      g2D.setStroke(widePen);                    // Use wide pen
      g2D.rotate(hourAngle);                     // Rotate to hour position
      g2D.draw(hourHand);                        // and draw hand

      // Draw minute hand
      g2D.setTransform(transform);               // Reset transform
      g2D.rotate(minuteAngle);                   // Rotate to minute position
      g2D.draw(minuteHand);                      // and draw hand

      // Draw second hand
      g2D.setStroke(narrowPen);                  // Use narrow pen
      g2D.setTransform(transform);               // Reset transform
      g2D.rotate(secondAngle);                   // Rotate to second position
      g2D.draw(secondHand);                      // and draw hand

      g2D.setPaint(Color.white);                 // Center color
      g2D.draw(center);                          // Draw center
    }
    
    Line2D.Double hourHand;
    Line2D.Double minuteHand;
    Line2D.Double secondHand;
    Ellipse2D.Double center;
    final Color CLEAR = new Color(0,0,0,0);
  }

  // Class defining the static face of the clock
  class ClockFace extends JPanel
  {
    // Creates a clock face with the given diameter
    public ClockFace(int diameter)
    {
      this.diameter = diameter;
      face = new Ellipse2D.Double();  
      hourMark = new Line2D.Double(0, -diameter*0.38,
                                   0, -diameter*0.48); 
      setOpaque(false);                         // Set panel transparent     
    }
    
    public void paint(Graphics g)
    {
      Dimension size = getSize();
      face.setFrame((size.width-diameter)/2,    // Set the size
                    (size.height-diameter)/2,   // of the face centered
                      diameter, diameter);      // and of the given diameter

      Graphics2D g2D = (Graphics2D)g;

      // Clear the panel
      g2D.setPaint(CLEAR);                      // Transparent color
      g2D.fillRect(0,0,size.width,size.height); // Fill the background

      g2D.setPaint(Color.lightGray);            // Face color
      g2D.fill(face);                           // Fill the clock face
      g2D.setPaint(Color.darkGray);             // Face outline color
      g2D.setStroke(widePen);                   // Use wide pen
      g2D.draw(face);                           // Draw face outline

      // Move origin to center of face
      g2D.translate(size.width/2, size.height/2);
    
      // Paint hour marks
      for(int i = 0 ; i<12 ; i++)
      {
        if(i%3 == 0)
          g2D.setStroke(widePen);              // Wide pen each quarter position
        else 
          g2D.setStroke(narrowPen);            // otherwise narrow pen

        g2D.draw(hourMark);                    // Draw the hour mark
        g2D.rotate(TWO_PI/12.0);               // Rotate to next mark
      }
    }

    int diameter;                              // Face diameter
    Ellipse2D.Double face;                     // The face
    Line2D.Double hourMark;                    // Mark for hours  
    final Color CLEAR = new Color(0,0,0,0);    // Transparent color
  }

  int seconds, minutes, hours;         // The current time

  // Increment the time by one second
  public void increment()
  {
     if(++seconds>= 60)                // If seconds reach 60
       ++minutes;                      // ...increment minutes

     if(minutes>=60)                   // If minutes reach 60
       ++hours;                        // ...increment hours

     seconds %= 60;                    // Seconds 0 to 59
     minutes %= 60;                    // Minutes 0 to 59
     hours %= 12;                      // Hours 0 to 11
    }

  BasicStroke widePen = new BasicStroke(3.0f,                         // Line width
                                      BasicStroke.CAP_ROUND,          // End cap
                                      BasicStroke.JOIN_MITER);        // Line join
  BasicStroke narrowPen = new BasicStroke(1.0f,
                                      BasicStroke.CAP_ROUND,
                                      BasicStroke.JOIN_MITER);

  final double TWO_PI = 2.0*Math.PI;
  final int ONE_SECOND = 1000;        // One second in milliseconds
  java.util.Timer timer;              // Timer to control the clock

  ClockFace clock;                    // The static clock face
  Hands hands;                        // The moving hands on the clock
}
