import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

public class FaderApplet extends JApplet
{
  public void init()
  {
    // Get parameters - if any
    String fade = getParameter("fadeTime");
    if(fade != null)
      fadeTime = Integer.parseInt(fade);
    String fps = getParameter("frameRate");
    if(fps != null)
      frameRate = Integer.parseInt(fps);

    maxCount = frameRate*fadeTime;               // Count of steps to complete fade
    imagePanel = new ImagePanel(getSize());
    getContentPane().add(imagePanel);

    composite = AlphaComposite.SrcOver;
  }

  // Parameter information for anyone that needs it
  public String[][] getParameterInfo()
  {
    String[][] info = {
             {"fadeTime" , "integer", "time to complete fade in seconds"},
             {"frameRate", "integer", "frames per second"               }
                      };
    return info;
  }

  public void start()
  {
    timer = new java.util.Timer(true);                     // Timer to run clock        
    count = maxCount;                                      // Set repaint counter
    alphaStep = 1.0f/count;
    long frameInterval = ONE_SECOND/frameRate;

    // Use fixed-delay execution to get smooth fade
    timer.schedule(new java.util.TimerTask()
                   {
                      public void run()
                      {
                        imagePanel.repaint();              // Repaint the image

                       // Update alpha composite for next frame
                       if(count ==maxCount)
                         countDelta = -1;
                       else if(count == 0)
                         countDelta = 1;
                       count += countDelta;
                       composite = AlphaComposite.getInstance(
                                               AlphaComposite.SRC_OVER,count*alphaStep);
                      }
                      
                      int countDelta = -1;      // Anonymous class member - count incr.
                   },
                    0,                                     // ...starting now
                    frameInterval);                        // Repaint interval
  }

  public void stop()
  {
    timer.cancel();
  }

  class ImagePanel extends JPanel
  {
    // Panel creates its own image from an image icon
    public ImagePanel(Dimension size)
    {
      ImageIcon icon = new ImageIcon("Images/wrox_logo.gif");
      image = icon.getImage();

      // Create a scaled image to fit within the size
      image = image.getScaledInstance(4*size.width/5, 4*size.height/5,
                                                                   Image.SCALE_SMOOTH);
      // Wait for scaled image to load
      MediaTracker tracker = new MediaTracker(this);
      tracker.addImage(image,0);                    // Image to track
      try
     {
      tracker.waitForID(0);
      }
      catch(InterruptedException e)
      {
      System.out.println(e);                       // Exception...
      System.exit(1);                              // ...so abandon ship!
      }
    }

    public void paint(Graphics g)
    {
      Graphics2D g2D = (Graphics2D)g;
      Dimension size = getSize();                    // Get panel size
      g2D.setPaint(Color.lightGray);                 // Background color
      g2D.fillRect(0,0,size.width,size.height);      // fill the panel
      g2D.setComposite(composite);                   // Set current alpha

      // Scale and draw image
      g2D.drawImage(image,                           // Image to be drawn
                    size.width/10,size.height/10,    // Image position inset
                    null);                           
    }

    // ImagePanel data members
    Image image;                 // The image
    int imageWidth;              // and its width
    int imageHeight;             // and height
  }

   // Applet data members
   final int ONE_SECOND = 1000;  // One second in milliseconds
   int frameRate = 20;           // Default fade change frequency frames per sec
   int fadeTime = 3;             // Default time to fade completely in seconds
   int count;                    // Repaint cycle counter
   int maxCount;

   ImagePanel imagePanel;        // Panel displaying the image
   AlphaComposite composite;     // Alpha value for the image
   float alphaStep;              // Alpha increment for fade step
   java.util.Timer timer;        // Timer to control fading  
}