import javax.swing.*;
import java.applet.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;

public class PlaySounds extends JFrame   
{
  public PlaySounds()
  {
    // Set up the window
    setDefaultCloseOperation(EXIT_ON_CLOSE); 
    setTitle("Sound File Player");
    setSize(250,100);             // Big enough to show the title bar & components

    // Get the sounds file names from current directory
    File currentDir = new File(System.getProperty("user.dir"));
    try
    {
      currentDirURL = currentDir.toURL();             // Get the directory URL
    }
    catch(MalformedURLException ex)
    {
      System.err.println("Bad URL  terminating...\n"+ex);
      System.exit(1);
    }

    FilenameFilter filter = new FilenameFilter()  // Filter for sound file types
     {
      public boolean accept(File directory, String filename)
       { 
        String name = filename.toLowerCase();
        return name.endsWith(".au")||name.endsWith(".aif")||name.endsWith(".wav");
       }
      };
    String soundFiles[] = currentDir.list(filter);      // Get all the file names
    if(soundFiles == null||soundFiles.length == 0)      // and check we got some
    {
      System.err.println("No sound files  terminating...");
      System.exit(1);
    }
    soundChoice = new JComboBox(soundFiles);            // Dropdown list box of files
    try
    {                            // Make a clip from the first file in the list
     clip = Applet.newAudioClip(new URL(currentDirURL,soundFiles[0]));
    }
    catch(MalformedURLException ex)
    {
      System.err.println("Bad URL  terminating...\n"+ex);
      System.exit(1);
    }

    // Add a listener for new sound file chosen
    soundChoice.addActionListener(new ActionListener()
     {
      public void actionPerformed(ActionEvent e)
      {
       try 
       {                           // create the clip from the file chosen
        clip = Applet.newAudioClip(new URL(currentDirURL,
                                  (String)soundChoice.getSelectedItem()));
       }
       catch(MalformedURLException ex)
       {
        System.err.println("Bad URL  terminating...\n"+ex);
        System.exit(1);
       }
      }
     }
    );

    // Set up the PLAY button to play the current clip
    JButton play = new JButton("PLAY");
    play.addActionListener(new ActionListener()   // Listen for button selection
     {
      public void actionPerformed(ActionEvent e)
      {
       clip.play();
      }
     }
    );

    // Add the combobox and the button to the application window
    Container content = getContentPane();
    content.add(soundChoice);
    content.add(play,BorderLayout.SOUTH);
  }

  public static void main(String args[])
  {
    PlaySounds player = new PlaySounds();   // Create the application object
    player.setVisible(true);                // and show the window
  }

  private URL currentDirURL;             // URL for the current directory
  private JComboBox soundChoice;         // Combobox for choosing the file to play
  private AudioClip clip;                // The current sound clip
}
