import javax.swing.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.sound.sampled.*;

public class SoundPlayer extends JFrame
{
  public SoundPlayer()
  {
    setDefaultCloseOperation(EXIT_ON_CLOSE); 
    setTitle("Sound File Player");
    setSize(250,100);

    // Get the sounds file names from current directory
    currentDir = new File(System.getProperty("user.dir"));
    FilenameFilter filter = new FilenameFilter()
                          {
                            public boolean accept(File directory, String filename)
                             { 
                               String name = filename.toLowerCase();
                               return name.endsWith(".au")|| 
                                      name.endsWith(".aif")||
                                      name.endsWith(".wav");
                                 }
                                };
    String soundFiles[] = currentDir.list(filter);
    if(soundFiles == null||soundFiles.length == 0)
    {
      JOptionPane.showMessageDialog(this, "No sound files  terminating...",
                                 "Sound Files Error", JOptionPane.ERROR_MESSAGE);
      System.exit(1);
    }

    // Create the combobox displaying the list of sound files
    soundChoice = new JComboBox(soundFiles); 
    soundChoice.setSelectedIndex(0);          // Select the 1st file
    oldFilename = soundFiles[0];              // save the selected file name

    player = new ClipPlayer(soundFiles[0]);   // Create a clip player

    // Process selections from the list of files in the combobox
    soundChoice.addActionListener(new ActionListener()
                            {
                               public void actionPerformed(ActionEvent e)
                               {player.newSound((String)soundChoice.getSelectedItem());
                             }
                            }
                                 );

    // Set up the PLAY button to play the current clip
    play = new JButton("PLAY");
    play.addActionListener(new ActionListener()
                              {
                                public void actionPerformed(ActionEvent e)
                                {
                                  if(e.getActionCommand().equals("PLAY"))
                                  {
                                    player.play();
                                    play.setText("STOP");
                                  }
                                  else
                                  {
                                    player.stop();
                                    play.setText("PLAY");
                                  }
                                }
                               });

    Container content = getContentPane();
    content.add(soundChoice);
    content.add(play,BorderLayout.SOUTH);
    setVisible(true);
  }

  public static void main(String[] args)
  {
    SoundPlayer soundPlayer = new SoundPlayer();
  }

  // Inner class defining a clip player
  public class ClipPlayer
  {

    public ClipPlayer(String filename)
    {
      newSound(filename);
    }

  public void newSound(String filename)
  {
    File soundFile = new File(currentDir, filename);  // File object for filename

    try
    {
      AudioInputStream source = AudioSystem.getAudioInputStream(soundFile);
      DataLine.Info clipInfo = new DataLine.Info(Clip.class, source.getFormat());
      if(AudioSystem.isLineSupported(clipInfo))
      {
        // Create a local clip to avoid discarding the old clip
        Clip newClip = (Clip)AudioSystem.getLine(clipInfo);   // Create the clip
        newClip.open(source);

        // Deal with previous clip
        if(clip != null)
        {
          if(clip.isActive())                 // If it's active
          {
            clip.stop();                      // ...stop it
            play.setText("PLAY");             // ..and reset the button label
          }
          if(clip.isOpen())                   // If it's open...
            clip.close();                     // ...close it.
        }
        clip = newClip;                       // We have a clip, so discard old
        oldFilename = filename;               // Save the filename
      }
      else
      {
        JOptionPane.showMessageDialog(null, "Clip not supported", 
                               "Clip NotSupported", JOptionPane.WARNING_MESSAGE);
        soundChoice.setSelectedItem(oldFilename);
      }
    }
    catch(UnsupportedAudioFileException e)
    {
      JOptionPane.showMessageDialog(null, "File not supported",
                           "Unsupported File Type", JOptionPane.WARNING_MESSAGE);
    }
    catch(LineUnavailableException e)
    {
      JOptionPane.showMessageDialog(null, "Clip not available", "Clip Error",
                                    JOptionPane.WARNING_MESSAGE);
    }
    catch(IOException e)
    {
      JOptionPane.showMessageDialog(null, "I/O error creating clip", "Clip Error",
                                    JOptionPane.WARNING_MESSAGE);
    }
  }

    // Play the clip
    public void play()
    {
      if(clip == null)
      {
        JOptionPane.showMessageDialog(null, "No Clip available", 
                                      "Play Problem",JOptionPane.WARNING_MESSAGE);
        return;
      }
      clip.loop(clip.LOOP_CONTINUOUSLY);    
    }

    // Stop playing the clip
    public void stop()
    {
      if(clip != null)
        clip.loop(0);
    }

    private Clip clip;                    // The sound clip
  }

  private File currentDir;                // Current directory
  private JComboBox soundChoice;          // Dropdown list of files
  private JButton play;                   // PLAY button
  private String oldFilename;             // Last selected file nmae

  private ClipPlayer player;              // The clip player
}
