import javax.swing.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import	javax.sound.sampled.*;

public class NewSoundPlayer extends JFrame
                            implements Runnable
{
  public NewSoundPlayer()
  {
    setDefaultCloseOperation(EXIT_ON_CLOSE); 
    setTitle("Sound File Player");
    setSize(250,100);

    // Get the sounds file names from current directory
    currentDir = new File(System.getProperty("user.dir"));
    FilenameFilter filter = new FilenameFilter()
                         {
                            public boolean accept(File directory, String filename)
                           { 
                             String name = filename.toLowerCase();
                             return name.endsWith(".au")|| 
                                    name.endsWith(".aif")||
                                    name.endsWith(".wav");
                           }
                          };
    String soundFiles[] = currentDir.list(filter);
    if(soundFiles == null||soundFiles.length == 0)
    {
      JOptionPane.showMessageDialog(this, "No sound files  terminating...",
                                 "Sound Files Error", JOptionPane.ERROR_MESSAGE);
      System.exit(1);
    }
    soundChoice = new JComboBox(soundFiles);
    soundChoice.setSelectedIndex(0);
    newSound(soundFiles[0]);
    oldFilename = soundFiles[0];    

    soundChoice.addActionListener(new ActionListener()
                              {
                                public void actionPerformed(ActionEvent e)
                                {
                                     newSound((String)soundChoice.getSelectedItem());
                                }
                              }
                                 );

    // Set up the PLAY button to play the current sound file
    play = new JButton("PLAY");
    play.addActionListener(new ActionListener()
                              {
                                public void actionPerformed(ActionEvent e)
                                {
                                  if(e.getActionCommand().equals("PLAY"))
                                  {
                                    startPlay();
                                    play.setText("STOP");
                                  }
                                  else
                                  {
                                    stopPlay();
                                    play.setText("PLAY");
                                  }
                                }
                               });
    Container content = getContentPane();
    content.add(soundChoice);
    content.add(play,BorderLayout.SOUTH);
    setVisible(true);
  }

  public static void main(String[] args)
  {
    NewSoundPlayer soundPlayer = new NewSoundPlayer();
  }

  public void newSound(String filename)
  {
    File soundFile = new File(currentDir, filename);

    // We may have played a file already
    if(sourceLine != null)                 // If we have a line
    {
      if(sourceLine.isActive())            // ... and it is still active...
        sourceLine.stop();                 // ...stop it
      play.setText("PLAY");                // Ensure button is PLAY
    }

    // Now try for a stream and a line
    try
    {
      AudioInputStream newSource = AudioSystem.getAudioInputStream(soundFile);

      if(newSource.markSupported())              // If we can mark the stream...
        newSource.mark(Integer.MAX_VALUE);       // mark the start for later reset

      AudioFormat format = newSource.getFormat(); // Get the audio format
      DataLine.Info sourceInfo = new DataLine.Info(SourceDataLine.class, format);
      if(AudioSystem.isLineSupported(sourceInfo)) // If the line type is supported
      {                                           // Get a new line
        sourceLine = (SourceDataLine)AudioSystem.getLine(sourceInfo);
        bufferSize = (int)(format.getFrameSize()*format.getFrameRate()/2.0f);
        sourceLine.open(format, bufferSize);      // Open the line
        source = newSource;                       // New line is OK so save it
        soundData = new byte[bufferSize];         // Create the buffer for read
        oldFilename = filename;                   // Save the current file name
      }
      else
      {
        JOptionPane.showMessageDialog(null, "Line not supported", 
                                 "Line NotSupported",JOptionPane.WARNING_MESSAGE);
         soundChoice.setSelectedItem(oldFilename); // Restore the old selection
      }
    }
    catch(UnsupportedAudioFileException e)
    {
      JOptionPane.showMessageDialog(null, "File not supported",
                             "Unsupported File Type",JOptionPane.WARNING_MESSAGE);
      soundChoice.setSelectedItem(oldFilename);
    }
    catch(LineUnavailableException e)
    {
      JOptionPane.showMessageDialog(null, "Line not available", "Line Error",
                                    JOptionPane.WARNING_MESSAGE);
      soundChoice.setSelectedItem(oldFilename);
    }
    catch(IOException e)
    {
      JOptionPane.showMessageDialog(null, "I/O Error creating stream", 
                                    "I/O Error",JOptionPane.WARNING_MESSAGE);
      soundChoice.setSelectedItem(oldFilename);
    }
  }

  // Start playing the current file
  public void startPlay()
  {
    if(sourceLine == null)                      // Verify we have a line
    {
      JOptionPane.showMessageDialog(null, "No line available", "Play Problem",
                                    JOptionPane.WARNING_MESSAGE);
      return;
    }
    thread = new Thread(this);                  // Create the playing thread
    playing = true;                             // Set the control to true
    thread.start();                             // Start the thread
  }

  
  // Stop playing the current file
  public void stopPlay()
  {
    playing = false;
  }

  
  // The playing thread
  public void run()
  {
    sourceLine.start();                       // Start the line    
    int byteCount = 0;                        // Bytes read 
    try
    {
      while(playing)                          // Continue while true
      {
       byteCount = source.read(soundData, 0, soundData.length); // Read the stream

       if(byteCount == -1)                   // If it's the end of input
       {
         if(source.markSupported())          // ...and we can reset the stream...
           source.reset();                   // ...put it back to the start
         sourceLine.drain();                 // Play what is left in the buffer
         playing = false;                    // Reset the thread control
         break;                              // then stop playing
       }
       sourceLine.write(soundData,0, byteCount);  // Write the array to the line
      }
    }
    catch(IOException e)                      // For the stream read operation
    {
      System.err.println(e);
    }
    sourceLine.stop();                        // Stop the line
    play.setText("PLAY");                     // Reset the button text
  }


  private File currentDir;                            // Current directory
  private String oldFilename;                         // Last selected file name
  private JComboBox soundChoice;                      // Dropdown list of files
  private JButton play;                               // PLAY button
  private AudioInputStream source;                    // Stream for the sound file
  private SourceDataLine sourceLine;                  // The speaker output line
  private byte[] soundData;                           // Buffer to hold samples
  private int bufferSize;                             // Buffer size in bytes
  private Thread thread;                              // Playing thread
  private boolean playing = false;                    // Thread control
}
