import javax.swing.*;
import javax.swing.border.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.sound.sampled.*;

public class Recorder extends JFrame
                      implements Runnable
{
  public static void main(String[] args)
  {
    Recorder recorder = new Recorder();
  }
  
  public Recorder()
  {
    setDefaultCloseOperation(EXIT_ON_CLOSE); 
    setTitle("Sound Recorder");
    setSize(250,200);

   currentDir = new File(System.getProperty("user.dir")); // Get current directory
   
   // Create a panel for the file name
   JPanel filenamePane = new JPanel(new GridLayout(0,1));
   CompoundBorder border = BorderFactory.createCompoundBorder(
                         BorderFactory.createEmptyBorder(5,5,5,5),
                         BorderFactory.createRaisedBevelBorder());
   filenamePane.setBorder(BorderFactory.createCompoundBorder(border,
                             BorderFactory.createEmptyBorder(5,5,5,5)));
   
   if((fileOut = getNewFile())==null)                      // Get a new file
   {
     System.err.println("Cannot create file");
     System.exit(1);
   }
   
   filenameLabel = new JLabel(fileOut.getName(), SwingConstants.CENTER);
   stopColor = filenameLabel.getForeground();
   filenamePane.add(filenameLabel);
   Container content = getContentPane();
   content.add(filenamePane);
   
   // Add the Record/Stop button
   record = new JButton("RECORD");
   record.setBorder(border);
   record.addActionListener(new ActionListener()
                            {
                              public void actionPerformed(ActionEvent e)
                              {
                                if(e.getActionCommand().equals("RECORD"))
                                {
                                  record.setText("STOP");
                                  startRecording();
                                }
                                else
                                {
                                  stopRecording();
                                  record.setText("RECORD");
                                }
                              }
                            }
                            );
    content.add(record,BorderLayout.SOUTH);

    setVisible(true);
  }
  
  // Create a new file
  File getNewFile()
  {
    File file = null;
    try
    {
      do
      {
        soundFileName = filename + (filenameSuffix++) + 
                                        '.' + fileType.getExtension();
        file = new File(currentDir, soundFileName);
      }while(!file.createNewFile());
      if(!file.isFile())
      {
          System.out.println("File not created: " + file.getName());
          return null;
      }
    }
    catch(IOException e)
    {
      System.out.println(e);
      return null;
    }
    return file;

  }
  
  public void startRecording()
  {
    DataLine.Info info = new DataLine.Info(TargetDataLine.class, format);
    if(!AudioSystem.isLineSupported(info))
    {
      System.out.println("Line not supported" + info);
      record.setEnabled(false);  
      return; 
    }
    try
    {
      mike = (TargetDataLine)AudioSystem.getLine(info);
      mike.open(format, mike.getBufferSize());
    }
    catch(LineUnavailableException e)
    {
      System.out.println("Line not available" + e);
      record.setEnabled(false);
      return;
    }
    if(fileOut.length()>0)
    {    
      fileOut = getNewFile();
      filenameLabel.setText(fileOut.getName());
    }
    filenameLabel.setForeground(recordColor);
    filenameLabel.repaint();
    thread = new Thread(this);
    thread.start();

  }

  public void stopRecording()
  {
    filenameLabel.setForeground(stopColor);
    filenameLabel.repaint();
    mike.stop();
    mike.close();

  }

  public void run()
  {
    AudioInputStream sound = new AudioInputStream(mike);  // Microphone stream
    mike.start();                                         // Start input
    try
    {
      AudioSystem.write(sound, fileType, fileOut);        // Write input to file
    }
    catch(IOException e)
    {
      System.out.println(e);
    }

  }
  

  private JLabel filenameLabel;                  // Displays the current file name
  private JButton record;                        // Record/Stop button


  private File currentDir;                       // Current directory
  private File fileOut;                          // Current sound file

  private String filename = "samples";               // Basic file name
  private int filenameSuffix = 0;                    // Name differentiator
  private String soundFileName;                      // The complete file name

  final Color recordColor = Color.red;           // Recording state color
  Color stopColor;                               // Stop state color

  // Sound file type
  private AudioFileFormat.Type fileType = AudioFileFormat.Type.WAVE;

  // Our chosen audio format  we could use a different format
  // as long as there is a line to support it
  final int MONO = 1;
  private  AudioFormat format = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED,
                                                44100, 16, MONO, 2, 44100, true);

  private TargetDataLine mike;                            // The microphone input
  private Thread thread;                                  // The recording thread

}
