import javax.swing.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.sound.midi.*;

public class MidiPlayer extends JFrame
{
  public MidiPlayer()
  {
    setDefaultCloseOperation(EXIT_ON_CLOSE); 
    setTitle("Midi File Player");
    setSize(250,100);

    // Set up the PLAY button to play the current midi file
    play = new JButton("PLAY");
    play.addActionListener(new ActionListener()
                              {
                                public void actionPerformed(ActionEvent e)
                                {
                                  if(e.getActionCommand().equals("PLAY"))
                                  {
                                    startPlay();
                                    play.setText("STOP");
                                  }
                                  else
                                  {
                                    stopPlay();
                                    play.setText("PLAY");
                                  }
                                }
                               });

    // Get the MIDI file names from current directory
    currentDir = new File(System.getProperty("user.dir"));
    FilenameFilter filter = new FilenameFilter()
                         {
                            public boolean accept(File directory, String filename)
                            { 
                              String name = filename.toLowerCase();
                              return name.endsWith(".mid"); 
                            }
                         };
    String midiFiles[] = currentDir.list(filter);
    if(midiFiles == null || midiFiles.length == 0)
    {
      JOptionPane.showMessageDialog(null, "No midi files  terminating...",
                                   "No Playable Files",JOptionPane.ERROR_MESSAGE);
      System.exit(1);
    }
    try
    {
      sequencer = MidiSystem.getSequencer();           // Get a sequencer...
      sequencer.open();                                // ...and open it.
    }
    catch(MidiUnavailableException e)
    {
JOptionPane.showMessageDialog(null, "No sequencer available - terminating.",
                              "Midi System Error", JOptionPane.ERROR_MESSAGE);
        System.exit(1);
    }
    
    midiChoice = new JComboBox(midiFiles);
    midiChoice.setSelectedIndex(0);
    newSequence(midiFiles[0]);
    oldFilename = midiFiles[0];
    
    midiChoice.addActionListener(new ActionListener()
                              {
                                public void actionPerformed(ActionEvent e)
                              {newSequence((String)midiChoice.getSelectedItem());
                                }
                              }
                            );

    Container content = getContentPane();
    content.add(midiChoice);
    content.add(play,BorderLayout.SOUTH);
    setVisible(true);
  }

  public static void main(String[] args)
  {
    MidiPlayer midiPlayer = new MidiPlayer();
  }
  
  void newSequence(String filename)
  {
    File midiFile = new File(currentDir, filename);
    try
    {
      sequence = MidiSystem.getSequence(midiFile);
      sequencer.setSequence(sequence);
      play.setText("PLAY");
      oldFilename = filename;
    }
    catch(InvalidMidiDataException e)
    {
      JOptionPane.showMessageDialog(null, "File "+filename+
               " contains invalid data","File Error", JOptionPane.ERROR_MESSAGE);
      midiChoice.setSelectedItem(oldFilename);
    }
    catch(IOException e)
    {
      JOptionPane.showMessageDialog(null, "I/O Error "+filename, 
                           "Sequence Creation Error", JOptionPane.ERROR_MESSAGE);
      midiChoice.setSelectedItem(oldFilename);
    }
  }
  
  void startPlay()
  {
    sequencer.start();
  }
  
  void stopPlay()
  {
    sequencer.stop();
  }

  private File currentDir;              // Current directory
  private String oldFilename;           // Last selected file nmae
  private JComboBox midiChoice;         // Dropdown list of files
  private JButton play;                 // PLAY button
  private Sequencer sequencer;          // Plays a Midi sequence
  private Sequence sequence;            // The Midi sequence         
}
