import java.sql.*;

public class StatementTest
{
    public StatementTest() throws SQLException, ClassNotFoundException
    {
      driverName = "sun.jdbc.odbc.JdbcOdbcDriver";
      sourceURL = "jdbc:odbc:technical_library";

      Class.forName (driverName);
      databaseConnection = DriverManager.getConnection(sourceURL);
    }

    public void doStatement() throws SQLException
    {
      Statement myStatement = databaseConnection.createStatement();
      ResultSet myResults = myStatement.executeQuery("SELECT authid, lastname, firstname FROM authors");

      showResults(myResults);
    }

    public void doPreparedStatement() throws SQLException
    {
      PreparedStatement myStatement = databaseConnection.prepareStatement("SELECT authid, lastname, firstname FROM authors");
      ResultSet myResults = myStatement.executeQuery();
      showResults(myResults);
    }

    public void showResults(ResultSet myResults) throws SQLException
    {
      // Retrieve ResultSetMetaData object from ResultSet
      ResultSetMetaData myResultMetadata = myResults.getMetaData();

      // How many columns were returned?
      int numColumns = myResultMetadata.getColumnCount();

      System.out.println("-------------Query Results----------------");
      // Loop through the ResultSet and get data
      while(myResults.next())
      {
        for(int column = 1; column <= numColumns; column++)
          System.out.print(myResults.getString(column) + "\t");
        System.out.print("\n");
      }

      System.out.println("\n\n-----------Query Metadata----------------");
      System.out.println("ResultSet contains " + numColumns + " columns");
      for (int column = 1; column <= numColumns; column++)
      {
        System.out.println("Column " + column);
        // Print the column name
        System.out.println("\tcolumn\t\t:" + myResultMetadata.getColumnName(column));
        // Print the label name
        System.out.println("\tlabel\t\t:" + myResultMetadata.getColumnLabel(column));
        // Print the column's display size
        System.out.println("\tdisplay width\t:" + myResultMetadata.getColumnDisplaySize(column) +
                               " characters");
        // Print the column's type
        System.out.println("\tdata type:\t:" + myResultMetadata.getColumnTypeName(column));
      }
    }

  public static void main(String[] args)
  {
    StatementTest SQLExample;
    try
    {
      SQLExample = new StatementTest();
      SQLExample.doStatement();
      SQLExample.doPreparedStatement();
    }
    catch(SQLException sqle)
    {
      System.err.println("SQL Exception: " + sqle);
    }
    catch(ClassNotFoundException cnfe)
    {
      System.err.println(cnfe.toString());
    }
  }

  Connection databaseConnection;                 // Connection to the database
  String driverName;                             // Database driver name
  String sourceURL;                              // Database location
}